'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.EventTimeLineCache import EventTimeLineCached

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "Move" )
    deleteEventTimeLineInBase(connection, "Move isolated" )
    deleteEventTimeLineInBase(connection, "Move in contact" )


def reBuildEvent( connection, file, tmin=None, tmax=None, pool = None  ):
    ''' 
    Animal A is stopped (built-in event): revert the event to have move events
    Move social: animal A is moving and in contact with another animal.
    Move isolated: animal A is moving and not in contact with any other animal.
    ''' 
    
    pool = AnimalPool( )
    pool.loadAnimals( connection )

    if len(pool.animalDictionnary.keys()) == 1:
        # print('Only one animal in database.')
        '''if the animal has been tested alone, only the move isolated event will be computed.'''
        moveSourceTimeLine = {}

        animal = 1
        ''' Load source stop timeLine and revert it to get the move timeline
        If the animal is not detected, this will result in a move. To avoid this we mask with the detection.
        '''
        moveSourceTimeLine[animal] = EventTimeLine(connection, "Stop", animal, minFrame=tmin, maxFrame=tmax, inverseEvent=True)
        detectionTimeLine = EventTimeLine(connection, "Detection", animal, minFrame=tmin, maxFrame=tmax)
        moveSourceTimeLine[animal].keepOnlyEventCommonWithTimeLine(detectionTimeLine)

        # initialisation of a dic for isolated move
        moveIsolatedResult = {}

        ''' loop over eventlist'''
        for moveEvent in moveSourceTimeLine[animal].eventList:
            for t in range(moveEvent.startFrame, moveEvent.endFrame + 1):
                moveIsolatedResult[t] = True


        ''' save move isolated at the end of the complete process for animalA'''
        moveIsolatedResultTimeLine = EventTimeLine(None, "Move isolated", idA=animal, idB=None, idC=None, idD=None, loadEvent=False)
        moveIsolatedResultTimeLine.reBuildWithDictionnary(moveIsolatedResult)
        moveIsolatedResultTimeLine.endRebuildEventTimeLine(connection)


    else:
        # print('More than one animal in database.')
        isInContactSourceDictionnary = {}
        moveSourceTimeLine = {}
        moveIsolatedTimeLine = {}

        for animal in pool.animalDictionnary.keys():
            ''' Load source stop timeLine and revert it to get the move timeline
            If the animal is not detected, this will result in a move. To avoid this we mask with the detection.
            '''
            moveSourceTimeLine[animal] = EventTimeLine( connection, "Stop", animal, minFrame=tmin, maxFrame=tmax, inverseEvent=True )
            detectionTimeLine = EventTimeLine( connection, "Detection", animal, minFrame=tmin, maxFrame=tmax )
            moveSourceTimeLine[animal].keepOnlyEventCommonWithTimeLine( detectionTimeLine )
            
            #by default let's say that all move are isolated; next we will extract frames which are in contact from this time line.
            moveIsolatedTimeLine[animal] = moveSourceTimeLine[animal]
            
            ''' load contact dictionary with another animal '''
            for animalB in pool.animalDictionnary.keys():
                if animal == animalB:
                    # print('Same identity')
                    continue
                else:
                    isInContactSourceDictionnary[(animal, animalB)] = EventTimeLineCached( connection, file, "Contact", animal, animalB, minFrame=tmin, maxFrame=tmax ).getDictionnary()

        moveIsolatedDic = {}
        for animal in pool.animalDictionnary.keys():
            #initialisation of a dic for isolated move
            moveIsolatedDic[animal] = moveIsolatedTimeLine[animal].getDictionary()

            for animalB in pool.animalDictionnary.keys():
                # initialisation of a dic for move in contact for each individual of the cage
                framesToRemoveFromMoveIsolatedTimeLine = []
                moveSocialResult = {}
                if animal == animalB:
                    # print('Same identity')
                    continue
                else:
                    ''' loop over eventlist'''
                    for moveEvent in moveIsolatedTimeLine[animal].eventList:

                        ''' for each event we seek in t and search a match in isInContactDictionnary between animal and animalB '''
                        for t in range ( moveEvent.startFrame, moveEvent.endFrame+1 ) :
                            if t in isInContactSourceDictionnary[(animal, animalB)]:
                                moveSocialResult[t] = True
                                framesToRemoveFromMoveIsolatedTimeLine.append(t)

                    ''' save move social at the end of the search for each animal'''
                    # clean the dictionary of the move and stop events from frames that are overlapping with exclusive contacts
                    for t in framesToRemoveFromMoveIsolatedTimeLine:
                        moveIsolatedDic[animal].pop(t, None)
                        
                    moveSocialResultTimeLine = EventTimeLine( None, "Move in contact" , idA=animal , idB=animalB , idC=None , idD=None , loadEvent=False )
                    moveSocialResultTimeLine.reBuildWithDictionnary( moveSocialResult )
                    moveSocialResultTimeLine.endRebuildEventTimeLine(connection)

            ''' save move isolated at the end of the complete process for animalA'''
            moveIsolatedResultTimeLine = EventTimeLine(None, "Move isolated", idA=animal, idB=None, idC=None, idD=None, loadEvent=False)
            moveIsolatedResultTimeLine.reBuildWithDictionnary(moveIsolatedDic[animal])
            moveIsolatedResultTimeLine.endRebuildEventTimeLine(connection)


        
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Move" , tmin=tmin, tmax=tmax )
                       
    # print( "Rebuild event finished." )
        
    


'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
#from affine import Affine
import matplotlib.pyplot as plt
import matplotlib.lines as mlines

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "Floor sniffing" )


def reBuildEvent( connection, tmin, tmax , pool = None ): 
    '''
    Event Floor sniffing:
    - the animal is sniffing the floor
    '''
    
    pool = AnimalPool( )
    pool.loadAnimals( connection )
    pool.loadDetection( start = tmin, end = tmax )
    
    for animal in pool.animalDictionnary.keys():
        # print(pool.animalDictionnary[animal])
        
        eventName = "Floor sniffing"
        # print ( "A sniffs the floor")
        # print ( eventName )
                
        sniffFloorTimeLine = EventTimeLine( None, eventName , animal , None , None , None , loadEvent=False )
                
        result={}
        
        animalA = pool.animalDictionnary[animal]
        #print ( animalA )
        dicA = animalA.detectionDictionnary
            
        for t in dicA.keys():
            if (animalA.getBodySlope(t) == None):
                continue
            
            if (animalA.getBodySlope(t) >= -25 and animalA.getBodySlope(t) <= -15):
                #print("floor sniffing")
                result[t] = True
                
        
        sniffFloorTimeLine.reBuildWithDictionnary( result )
                
        sniffFloorTimeLine.endRebuildEventTimeLine(connection)
    
        
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Floor sniffing" , tmin=tmin, tmax=tmax )

    # print( "Rebuild event finished." )
        
        
        
        
        
        
        
    


'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
#from affine import Affine
import matplotlib.pyplot as plt
import matplotlib.lines as mlines
from lmtanalysis.EventTimeLineCache import EventTimeLineCached

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "Water Zone" )
    deleteEventTimeLineInBase(connection, "Water Stop" )


def reBuildEvent( connection, file, tmin=None, tmax=None, pool = None ): 
    
    ''' use the pool provided or create it'''
    if ( pool == None ):
        pool = AnimalPool( )
        pool.loadAnimals( connection )
        pool.loadDetection( start = tmin, end = tmax, lightLoad=True )
    '''
    Event Water Zone
    - the animal is in the zone around the water source
    - the animal is stopped in this zone for 
    '''
    
    for animal in pool.animalDictionnary.keys():
        # print(pool.animalDictionnary[animal])
        
        eventName1 = "Water Zone"
        eventName2 = "Water Stop"
        # print ( "A is in the zone around the water source")
        # print ( eventName1 )
                
        waterZoneTimeLine = EventTimeLine( None, eventName1 , animal , None , None , None , loadEvent=False )
        waterStopTimeLine = EventTimeLine( None, eventName2 , animal , None , None , None , loadEvent=False )
        
        
        stopTimeLine = EventTimeLineCached( connection, file, "Stop", animal, minFrame=None, maxFrame=None )
        stopTimeLineDictionary = stopTimeLine.getDictionary()
                
        resultWaterZone={}
        resultWaterStop={}
        
        animalA = pool.animalDictionnary[animal]
        #print ( animalA )
        dicA = animalA.detectionDictionnary
            
        for t in dicA.keys():
            if (dicA[t].getDistanceToPoint(xPoint = 398, yPoint = 353) == None):
                continue
            
            #Check if the animal is entering the zone around the water point:
            if (dicA[t].getDistanceToPoint(xPoint = 398, yPoint = 353) <= MAX_DISTANCE_TO_POINT*2):
                resultWaterZone[t] = True
            
            #Check if the animal is drinking (the animal should be in a tight zone around the water point and be stopped):      
            if (dicA[t].getDistanceToPoint(xPoint = 398, yPoint = 353) <= MAX_DISTANCE_TO_POINT):
                if t in stopTimeLineDictionary.keys():
                    resultWaterStop[t] = True
                
        
        waterZoneTimeLine.reBuildWithDictionnary( resultWaterZone )
        waterZoneTimeLine.endRebuildEventTimeLine(connection)
        
        waterStopTimeLine.reBuildWithDictionnary( resultWaterStop )
        waterStopTimeLine.removeEventsBelowLength( maxLen = MIN_WATER_STOP_DURATION )    
        waterStopTimeLine.endRebuildEventTimeLine(connection)
    
        
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Water Point" , tmin=tmin, tmax=tmax )

    # print( "Rebuild event finished." )
        
        
        
        
        
        
        
    


'''
Created on 25 sept. 2018

@author: Fab
'''
from lmtanalysis.Event import EventTimeLine

eventCacheDico_={}
eventCacheEnable_ = True

def disableEventTimeLineCache():
    ''' If you are using a computer with small amount of RAM, it might be better to disable cache '''
    eventCacheDico_.clear()
    global eventCacheEnable_
    eventCacheEnable_ = False
    # print( "Cache event is disabled" )
    
    
def EventTimeLineCached( connection, file, eventName, idA=None, idB=None, idC=None, idD = None, minFrame=None, maxFrame=None, loadEventIndependently=False ):
            
    global eventCacheDico_
    
    if eventCacheEnable_ == True:
    
        #print ("Cache debug:", connection, file, eventName, "ids:" , idA, idB, idC, idD, "frames:", minFrame, maxFrame , loadEventWithoutOverlapCheck )
        
        
        if (file, eventName, idA, idB, idC, idD, minFrame, maxFrame, loadEventIndependently ) in eventCacheDico_: 
            eventTimeLine = eventCacheDico_[ file, eventName, idA, idB, idC, idD, minFrame, maxFrame, loadEventIndependently ]
            # print ( eventName , " Id(",idA ,",", idB, ",", idC, "," , idD , ") Loaded from cache (" , len( eventTimeLine.eventList ) , " records. )")
            return eventTimeLine

        
    eventTimeLine = EventTimeLine( connection, eventName , idA = idA, idB = idB, idC = idC, idD = idD, minFrame = minFrame, maxFrame = maxFrame , loadEventIndependently = loadEventIndependently )
    
    if eventCacheEnable_ == True:
        # print ( "Caching eventTimeLine")
        eventCacheDico_[ file, eventName, idA, idB, idC, idD, minFrame, maxFrame, loadEventIndependently ] = eventTimeLine
    # else:
    #     print( "Loading event from base (Cache event disabled)")
    
    return eventTimeLine
        
def flushEventTimeLineCache():
    eventCacheDico_.clear()


'''
Created on 6 sept. 2017

@author: Fab
'''

import math


class Rectangle:

    def __init__(self, pA , pB):
        self.pA = pA;
        self.pB = pB;
        
    def isPointInside(self , p ):
        
        if ( p.x >= self.pA.x and p.y >= self.pA.y and p.x <= self.pB.x and p.y <= self.pB.y ):
            return True
        return False
        
        
        


'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.EventTimeLineCache import EventTimeLineCached

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "Train2" )


def reBuildEvent( connection, file, tmin=None, tmax=None , pool = None ): 

    ''' use pool cache if available '''
    if ( pool == None ):
        pool = AnimalPool( )
        pool.loadAnimals( connection )
        pool.loadDetection( start = tmin, end = tmax )    
    
    '''
    two animals are following each others with nose-to-anogenital contacts
    animals are moving    
    '''
    
    #deleteEventTimeLineInBase(connection, "Train2" )
    

                
    contactHeadGenital = {}
    for animal in range( 1,pool.getNbAnimals()+1 ):
        for idAnimalB in range( 1 , pool.getNbAnimals()+1 ):
            if ( animal == idAnimalB ):
                continue
            contactHeadGenital[animal, idAnimalB] = EventTimeLineCached( connection, file, "Oral-genital Contact", animal, idAnimalB, minFrame=tmin, maxFrame=tmax )


    for animal in range( 1 , pool.getNbAnimals()+1 ):
        
        for idAnimalB in range( 1 , pool.getNbAnimals()+1 ):

            if( animal == idAnimalB ):
                continue
                            
            eventName = "Train2"        
            # print ( eventName )
            
            trainTimeLine = EventTimeLine( None, eventName , animal , idAnimalB, loadEvent=False )
            
            result={}
            
            dicAB = contactHeadGenital[ animal , idAnimalB ].getDictionnary()
            
            for t in dicAB.keys():
                speedA = pool.animalDictionnary[animal].getSpeed(t)
                speedB = pool.animalDictionnary[idAnimalB].getSpeed(t)
                        
                if ( speedA != None and speedB != None ):
                    if ( speedA > SPEED_THRESHOLD_HIGH and speedB > SPEED_THRESHOLD_HIGH ):
                        result[t]=True
            
            trainTimeLine.reBuildWithDictionnary( result )
            trainTimeLine.removeEventsBelowLength( 5 )            
            trainTimeLine.endRebuildEventTimeLine(connection)
    
        
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Train2" , tmin=tmin, tmax=tmax )

                
    # print( "Rebuild event finished." )
        
    
    
    


'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
#from affine import Affine
import matplotlib.pyplot as plt
import matplotlib.lines as mlines
    
def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "SideWalk" )

def projectVector( x1, y1, x2, y2 ):
    '''
    Project vector 1 on vector 2
    '''
    x = ( x1*x2 + y1*y2 ) * x2
    y = ( x1*x2 + y1*y2 ) * y2
    
    return x,y

    
def rotateVector( x1, y1 ):
    '''
    Rotate PI/2
    '''
    x = -y1
    y = x1
    return x, y

def getVectorFromAngle( angle ):
    
    x = math.cos(angle)
    y = math.sin(angle)
    return x,y

def normVector( x, y ):
    '''
    Norm of the vector
    '''
    norm = math.sqrt( x*x + y*y )
    return norm

def reBuildEvent( connection ): 
    '''
    Event SideWalk:

    - Side velocity is over a threshold

    '''
    timeWindow =  oneHour*2
        
    pool = AnimalPool( )
    pool.loadAnimals( connection )
    pool.loadDetection( start = 0, end = timeWindow )
    
    for animal in pool.animalDictionnary.keys():
        # print(pool.animalDictionnary[animal])
                    
        eventName = "SideWalk".format( animal )
                
        # print ( eventName )
                
        sideWalkTimeLine = EventTimeLine( None, eventName, animal , None, None, None, loadEvent=False )
                
        result={}
        
        animalA = pool.animalDictionnary[animal]
        dicA = animalA.detectionDictionnary
            
        for t in dicA.keys():
            speedX, speedY = pool.animalDictionnary[animal].getSpeedVector(t)
            
            if ( speedX == None or speedY == None ):
                continue
            
            if not pool.animalDictionnary[animal].getDetectionAt( t ).isHeadAndTailDetected():
                continue
                    
            angleA = pool.animalDictionnary[animal].getDirection(t)
            
            vectorSideX, vectorSideY = getVectorFromAngle( angleA + math.pi / 2 )
            
            xP, yP = projectVector( speedX , speedY , vectorSideX, vectorSideY )

            normSide = normVector( xP , yP )
            
            if ( normSide > 15 ):
                result[t]=True
            '''
            if ( t == 1128 ):
                print ( "speedX: " , speedX )
                print ( "speedY: " , speedY )

                print ( "angleA: " , angleA )
                print ( "sidex: " , vectorSideX )
                print ( "sidey: " , vectorSideY )
                print ( "xp: " , xP )
                print ( "yp: " , yP )
                print ( "norm: " , normSide )
            '''
        sideWalkTimeLine.reBuildWithDictionnary( result )

        #print("REMOVING SHORT EVENTS !")
        #sideWalkTimeLine.removeEventsBelowLength( 5 )
                
        # if (len( sideWalkTimeLine.eventList) == 0):
        #     print("no event")
        # else:
        #     print ( "Number of event: " , len( sideWalkTimeLine.eventList ) )
        #     print ( "Mean len of event: " , sideWalkTimeLine.getMeanEventLength() )
        #     print ( "first event frame: " , sideWalkTimeLine.timeLineList[0].startFrame )
        
        # print ( "Delete old entry in base: " + eventName )
        deleteEventTimeLineInBase(connection, eventName, animal, None, None, None ) #None is assumed as the name of the event contains the id description
        # print ( "Saving timeLine: " + eventName )
        sideWalkTimeLine.saveTimeLine(connection)
    
        
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Side Walk" )

    # print( "Rebuild event finished." )
        
    


'''
Created on 15 Feb. 2022

@author: Elodie
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.EventTimeLineCache import EventTimeLineCached
from lmtanalysis.BehaviouralSequencesUtil import exclusiveEventList, contactTypeList

def flush( connection ):
    ''' flush event in database '''
    for exclusiveEvent in exclusiveEventList[:-3]:
        deleteEventTimeLineInBase(connection, exclusiveEvent)

def reBuildEvent( connection, file, tmin=None, tmax=None , pool = None ):

    if ( pool == None ):
        pool = AnimalPool( )
        pool.loadAnimals( connection )

    ''' load the existing timelines for each pair of animals '''
    timeLine = {}
    for event in contactTypeList:
        timeLine[event] = {}
        for animal in range(1, pool.getNbAnimals() + 1):
            for idAnimalB in range(1, pool.getNbAnimals() + 1):
                if (animal == idAnimalB):
                    continue
                timeLine[event][animal, idAnimalB] = EventTimeLineCached( connection, file, event, animal, idAnimalB, minFrame=tmin, maxFrame=tmax )

    ''' create the timelines of the different exclusive contacts '''
    timeLineExclusive = {}
    for event in exclusiveEventList[:-3]:
        timeLineExclusive[event] = {}
        for animal in range(1, pool.getNbAnimals() + 1):
            for idAnimalB in range(1, pool.getNbAnimals() + 1):
                if (animal == idAnimalB):
                    continue
                timeLineExclusive[event][animal, idAnimalB] = EventTimeLine(None, event, animal, idAnimalB, loadEvent=False)

    #generate the dico of the different contact types
    contactDico = {}
    for event in contactTypeList:
        contactDico[event] = {}
        for animal in range(1, pool.getNbAnimals() + 1):
            for idAnimalB in range(1, pool.getNbAnimals() + 1):
                if (animal == idAnimalB):
                    continue
                contactDico[event][animal, idAnimalB] = timeLine[event][animal, idAnimalB].getDictionary(minFrame=tmin, maxFrame=tmax)


    # initiate the list of frames to remove from each timeline
    framesToRemove = {}
    for exclusiveEvent in exclusiveEventList[:-3]:
        framesToRemove[exclusiveEvent] = {}
        for animal in range(1, pool.getNbAnimals() + 1):
            for idAnimalB in range(1, pool.getNbAnimals() + 1):
                if (animal == idAnimalB):
                    continue
                framesToRemove[exclusiveEvent][animal, idAnimalB] = []

    # initiate the dico that will be used to reconstruct the exclusive timelines
    contactDicoExclusive = {}
    for exclusiveEvent in exclusiveEventList[:-3]:
        # print('############# ', exclusiveEvent)
        contactDicoExclusive[exclusiveEvent] = {}
        for animal in range(1, pool.getNbAnimals() + 1):
            for idAnimalB in range(1, pool.getNbAnimals() + 1):
                if (animal == idAnimalB):
                    continue
                contactDicoExclusive[exclusiveEvent][animal, idAnimalB] = {}

    # save the dictionary of contact types as exclusive contacts before cleaning them
    eventPairs = {}
    for event in contactTypeList:
        eventPairs[event] = event+' exclusive'

    for event in eventPairs.keys():
        contactDicoExclusive[eventPairs[event]] = contactDico[event]

    ###########################################################################
    # clean the timelines of contactTypes where different contactTypes are overlapping and are not intuitive
    for animal in range(1, pool.getNbAnimals() + 1):
        for idAnimalB in range(1, pool.getNbAnimals() + 1):
            if (animal == idAnimalB):
                continue
            eventA = 'Oral-oral Contact'
            eventB = 'Oral-genital Contact'
            for t in contactDico[eventA][animal, idAnimalB].keys():
                if t in contactDico[eventB][animal, idAnimalB].keys():
                    contactDicoExclusive['Other contact exclusive'][animal, idAnimalB][t] = True
                    framesToRemove[eventA+' exclusive'][animal, idAnimalB].append(t)
                    framesToRemove[eventB+' exclusive'][animal, idAnimalB].append(t)

            eventA = 'Oral-oral Contact'
            eventB = 'Passive oral-genital Contact'
            for t in contactDico[eventA][animal, idAnimalB].keys():
                if t in contactDico[eventB][animal, idAnimalB].keys():
                    contactDicoExclusive['Other contact exclusive'][animal, idAnimalB][t] = True
                    framesToRemove[eventA + ' exclusive'][animal, idAnimalB].append(t)
                    framesToRemove[eventB + ' exclusive'][animal, idAnimalB].append(t)

            eventA = 'Oral-oral Contact'
            eventB = 'Side by side Contact, opposite way'
            for t in contactDico[eventA][animal, idAnimalB].keys():
                if t in contactDico[eventB][animal, idAnimalB].keys():
                    contactDicoExclusive['Other contact exclusive'][animal, idAnimalB][t] = True
                    framesToRemove[eventA + ' exclusive'][animal, idAnimalB].append(t)
                    framesToRemove[eventB + ' exclusive'][animal, idAnimalB].append(t)

            eventA = 'Side by side Contact'
            eventB = 'Side by side Contact, opposite way'
            for t in contactDico[eventA][animal, idAnimalB].keys():
                if t in contactDico[eventB][animal, idAnimalB].keys():
                    contactDicoExclusive['Other contact exclusive'][animal, idAnimalB][t] = True
                    framesToRemove[eventA + ' exclusive'][animal, idAnimalB].append(t)
                    framesToRemove[eventB + ' exclusive'][animal, idAnimalB].append(t)

            eventA = 'Side by side Contact'
            eventB = 'Oral-genital Contact'
            for t in contactDico[eventA][animal, idAnimalB].keys():
                if t in contactDico[eventB][animal, idAnimalB].keys():
                    contactDicoExclusive['Other contact exclusive'][animal, idAnimalB][t] = True
                    framesToRemove[eventA + ' exclusive'][animal, idAnimalB].append(t)
                    framesToRemove[eventB + ' exclusive'][animal, idAnimalB].append(t)

            eventA = 'Side by side Contact'
            eventB = 'Passive oral-genital Contact'
            for t in contactDico[eventA][animal, idAnimalB].keys():
                if t in contactDico[eventB][animal, idAnimalB].keys():
                    contactDicoExclusive['Other contact exclusive'][animal, idAnimalB][t] = True
                    framesToRemove[eventA + ' exclusive'][animal, idAnimalB].append(t)
                    framesToRemove[eventB + ' exclusive'][animal, idAnimalB].append(t)


    ###########################################################################
    #exclude the side-side opposite contacts where animals are also in oral-genital sniffing
    for animal in range(1, pool.getNbAnimals() + 1):
        for idAnimalB in range(1, pool.getNbAnimals() + 1):
            if (animal == idAnimalB):
                continue

            for t in contactDico["Side by side Contact, opposite way"][animal, idAnimalB].keys():
                if t in contactDico['Oral-genital Contact'][animal, idAnimalB].keys():
                    if t not in contactDicoExclusive['Other contact exclusive'][animal, idAnimalB].keys():
                        framesToRemove['Side by side Contact, opposite way exclusive'][animal, idAnimalB].append(t)
                        framesToRemove['Oral-genital Contact exclusive'][animal, idAnimalB].append(t)
                        framesToRemove['Passive oral-genital Contact exclusive'][idAnimalB, animal].append(t)
                        contactDicoExclusive["Oral-genital and Side by side Contact, opposite way exclusive"][animal, idAnimalB][t] = True

                elif t in contactDico['Passive oral-genital Contact'][animal, idAnimalB].keys():
                    if t not in contactDicoExclusive['Other contact exclusive'][animal, idAnimalB].keys():
                        framesToRemove['Side by side Contact, opposite way exclusive'][animal, idAnimalB].append(t)
                        framesToRemove['Passive oral-genital Contact exclusive'][animal, idAnimalB].append(t)
                        framesToRemove['Oral-genital Contact exclusive'][idAnimalB, animal].append(t)
                        contactDicoExclusive['Oral-genital passive and Side by side Contact, opposite way exclusive'][animal, idAnimalB][t] = True

                else:
                    contactDicoExclusive["Side by side Contact, opposite way exclusive"][animal, idAnimalB][t] = True

    ###########################################################################
    # exclude the side-side contacts where animals are also in oral-oral sniffing
    for animal in range(1, pool.getNbAnimals() + 1):
        for idAnimalB in range(1, pool.getNbAnimals() + 1):
            if (animal == idAnimalB):
                continue

            for t in contactDico["Side by side Contact"][animal, idAnimalB].keys():
                if t in contactDico['Oral-oral Contact'][animal, idAnimalB].keys():
                    framesToRemove['Side by side Contact exclusive'][animal, idAnimalB].append(t)
                    framesToRemove['Oral-oral Contact exclusive'][animal, idAnimalB].append(t)
                    framesToRemove['Other contact exclusive'][animal, idAnimalB].append(t)
                    contactDicoExclusive["Oral-oral and Side by side Contact exclusive"][animal, idAnimalB][t] = True

                else:
                    contactDicoExclusive["Side by side Contact exclusive"][animal, idAnimalB][t] = True


    #############################################################
    # taking out the frames from the exclusive timelines
    for event in exclusiveEventList[:-3]:
        for animal in range(1, pool.getNbAnimals() + 1):
            for idAnimalB in range(1, pool.getNbAnimals() + 1):
                if (animal == idAnimalB):
                    continue
                for t in framesToRemove[event][animal, idAnimalB]:
                    contactDicoExclusive[event][animal, idAnimalB].pop(t, None)


    #####################################################
    #reduild all events based on dictionary
    for event in exclusiveEventList[:-3]:
        for animal in range(1, pool.getNbAnimals() + 1):
            for idAnimalB in range(1, pool.getNbAnimals() + 1):
                if (animal == idAnimalB):
                    continue

                timeLineExclusive[event][animal, idAnimalB].reBuildWithDictionnary(contactDicoExclusive[event][animal, idAnimalB])
                timeLineExclusive[event][animal, idAnimalB].endRebuildEventTimeLine(connection)


    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Exclusive oral-oral, oral-genital & side-side contacts" , tmin=tmin, tmax=tmax )
                       
    # print( "Rebuild event finished." )
        
    


'''

@author: Fab

Loads log of process performed on a lmtanalysis

CREATE TABLE "LOG" (
    `id`    INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE,
    `process`    TEXT,
    `version`    TEXT,
    `date`    TEXT,
    `tmin`    INTEGER,
    `tmax`    INTEGER
)

test in: 2478 USV event

'''
from datetime import date, datetime
from tabulate import tabulate

class Log:
    
    def __init__(self, id, process, version, date, tmin, tmax ):        
        
        self.id = id
        self.process = process
        self.version = version
        self.date = date
        self.tmin = tmin
        self.tmax = tmax
    
class TaskLogger:

    def __init__(self, connection ):        
        self.logList = []
        self.conn = connection
        
        self.createLogTableIfNeeded()
        
        self.loadLog()
        
    def createLogTableIfNeeded(self):
        
        try:
            cursor = self.conn.cursor()
            query = "SELECT ID, PROCESS, VERSION, DATE, TMIN, TMAX FROM LOG"
            cursor.execute( query )
            cursor.close()
        except:
            #create table
            # print("Log table does not exists, creating Log table")
            cursor = self.conn.cursor()
            query = "CREATE TABLE LOG ( id INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE, process TEXT, version TEXT,date TEXT,tmin INTEGER, tmax INTEGER)"
            cursor.execute( query )
            self.conn.commit()
            cursor.close()
    
    def loadLog(self):

        #print ( self.__str__(), ": Loading log" )

        self.logList.clear()
        cursor = self.conn.cursor()
        query = "SELECT ID, PROCESS, VERSION, DATE, TMIN, TMAX FROM LOG"
            
        #print( query )
        cursor.execute( query )
        
        rows = cursor.fetchall()
        cursor.close()    
        
        for row in rows:
            id = row[0]
            process = row[1]
            version = row[2]
            date = row[3]
            tmin = row[4]
            tmax = row[5]
            
            log = Log( id, process, version, date, tmin, tmax )
            
            self.logList.append( log )
        
        #print ( self.__str__(), " ", " {} log line loaded: " )

    def listLog(self):
        
        header = [ "ID", "Process", "version", "date Y-M-D H:M:S", "tmin", "tmax" ]
        data =[]
    
        for l in self.logList:
    
            data.append( [ l.id, l.process, l.version, l.date , l.tmin, l.tmax ])
    
        # print ( tabulate( data, header, tablefmt='orgtbl') )

    def addLog(self, process, version='0', date=None, tmin=0, tmax=-1 ):

        if date==None:
            
            date = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        
        cursor = self.conn.cursor()
        query = "INSERT INTO LOG( process,version,date,tmin,tmax) VALUES ( '{process}','{version}','{date}','{tmin}','{tmax}' );".format( process=process, version=version, date=date, tmin=tmin, tmax=tmax )
                    
        # print( query )
        cursor.execute( query )
        self.conn.commit()
        cursor.close()    
        
        self.loadLog()

    





import sys
sys.path.insert(1, "../")

import sqlite3
import os
from tqdm import tqdm
from time import *
import datetime
import matplotlib.pyplot as plt
import traceback
import numpy as np
import pandas as pd
import warnings
warnings.simplefilter(action='ignore')

from lmtanalysis.Util import *
from lmtanalysis.Animal import *
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.BuildEventNight import *
from lmtanalysis import BuildEventApproachContact, BuildEventOtherContact, BuildEventPassiveAnogenitalSniff, \
    BuildEventHuddling, BuildEventTrain3, BuildEventTrain4, BuildEventTrain2, BuildEventFollowZone, BuildEventRear5, \
    BuildEventCenterPeripheryLocation, BuildEventRearCenterPeriphery, BuildEventFloorSniffing, BuildEventSocialApproach, \
    BuildEventSocialEscape, BuildEventApproachContact, BuildEventOralOralContact, BuildEventApproachRear, \
    BuildEventGroup2, BuildEventGroup3, BuildEventGroup4, BuildEventOralGenitalContact, BuildEventStop, \
    BuildEventWaterPoint, BuildEventMove, BuildEventGroup3MakeBreak, BuildEventGroup4MakeBreak, BuildEventSideBySide, \
    BuildEventSideBySideOpposite, BuildEventDetection, BuildDataBaseIndex, BuildEventWallJump, BuildEventSAP, \
    BuildEventOralSideSequence, CheckWrongAnimal, CorrectDetectionIntegrity, BuildEventNest4, BuildEventNest3, \
    BuildEventGetAway
from psutil import virtual_memory
from tkinter.filedialog import askopenfilename
from lmtanalysis.TaskLogger import TaskLogger
from lmtanalysis.FileUtil import getFilesToProcess
from lmtanalysis.EventTimeLineCache import flushEventTimeLineCache, \
    disableEventTimeLineCache, EventTimeLineCached

# --------------------------------------------------------------------------------------------------------
# Start of rebuild
# --------------------------------------------------------------------------------------------------------

# Select databases to rebuild and create csv
files = getFilesToProcess()

count = 0

''' minT and maxT to process the analysis (in frame) '''
minT = 0

# maxT = 5000
maxT = 72 * oneHour
# maxT = (6+1)*oneHour
''' time window to compute the events. '''
# windowT = 1 * oneHour
windowT = 1 * oneDay
# windowT = 3*oneDay #int (0.5*oneDay)

USE_CACHE_LOAD_DETECTION_CACHE = True

class FileProcessException(Exception):
    pass

'''Few events must be created with others like Move which must be activated with Stop events because Move events are 
created in relation to Stop events '''

eventClassList = [
                # # BuildEventHuddling,
                BuildEventDetection,
                BuildEventStop,
                BuildEventMove,
                BuildEventOralOralContact,
                BuildEventOralGenitalContact,
                BuildEventSideBySide,
                BuildEventSideBySideOpposite,
                BuildEventTrain2,
                BuildEventTrain3,
                # BuildEventTrain4,
                BuildEventFollowZone,
                BuildEventRear5,
                BuildEventCenterPeripheryLocation,
                BuildEventRearCenterPeriphery,
                BuildEventSocialApproach,
                BuildEventGetAway,
                BuildEventSocialEscape,
                BuildEventApproachRear,
                BuildEventGroup2,
                BuildEventGroup3,
                # BuildEventGroup4,
                BuildEventGroup3MakeBreak,
                # BuildEventGroup4MakeBreak,
                # BuildEventWaterPoint,
                BuildEventApproachContact,
                # BuildEventWallJump,
                BuildEventSAP,
                BuildEventOralSideSequence,
                BuildEventNest3,
                # BuildEventNest4
                   ]

# eventClassList = [BuildEventStop]

# eventClassList = [BuildEventPassiveAnogenitalSniff, BuildEventOtherContact, BuildEventExclusiveSideSideNoseAnogenitalContact]
# eventClassList = [BuildEventApproachContact2]

'''eventClassList = [

                BuildEventDetection,
                BuildEventMove,
                BuildEventRear5,
                BuildEventCenterPeripheryLocation,
                BuildEventRearCenterPeriphery,
                BuildEventStop,
                BuildEventWaterPoint,
                BuildEventWallJump,
                BuildEventSAP
                   ]'''

def flushNightEvents(connection):
    ''' flush 'NIGHT' event in database '''
    # print("delete night in DBs ?")
    deleteEventTimeLineInBase(connection, "night")

def flushEvents( connection ):

    for ev in eventClassList:

        chrono = Chronometer( "Flushing event " + str(ev) )
        ev.flush( connection );

def processTimeWindow(connection, file, currentMinT, currentMaxT):
    CheckWrongAnimal.check(connection, tmin=currentMinT, tmax=currentMaxT)

    # Warning: enabling this process (CorrectDetectionIntegrity) will alter the database permanently
    # CorrectDetectionIntegrity.correct(connection, tmin=0, tmax=maxT)

    # BuildEventDetection.reBuildEvent(connection, file, tmin=currentMinT, tmax=currentMaxT)

    animalPool = None

    flushEventTimeLineCache()

    if (USE_CACHE_LOAD_DETECTION_CACHE):
        animalPool = AnimalPool()
        animalPool.loadAnimals(connection)
        animalPool.loadDetection(start=currentMinT, end=currentMaxT)

    for ev in eventClassList:
        chrono = Chronometer(str(ev))
        ev.reBuildEvent(connection, file, tmin=currentMinT, tmax=currentMaxT, pool=animalPool)

def process(file):

    mem = virtual_memory()
    availableMemoryGB = mem.total / 1000000000

    if availableMemoryGB < 10:
        disableEventTimeLineCache()

    chronoFullFile = Chronometer("File " + file)

    connection = sqlite3.connect(file)

    try:
        connection = sqlite3.connect(file)
        c = connection.cursor()
        query = "ALTER TABLE EVENT ADD METADATA TEXT";
        c.execute(query)
        connection.commit()

    except Exception as e:
        pass

    BuildDataBaseIndex.buildDataBaseIndex(connection, force=False)
    # build sensor data
    animalPool = AnimalPool()
    animalPool.loadAnimals(connection)

    currentT = minT
    try:
        flushEvents(connection)
        while currentT < maxT:
            currentMinT = currentT
            currentMaxT = currentT + windowT
            if (currentMaxT > maxT):
                currentMaxT = maxT
            chronoTimeWindowFile = Chronometer(
                "File " + file + " currentMinT: " + str(currentMinT) + " currentMaxT: " + str(currentMaxT));
            processTimeWindow(connection, file, currentMinT, currentMaxT)
            # chronoTimeWindowFile.printTimeInS()
            currentT += windowT

        TEST_WINDOWING_COMPUTATION = False

        if (TEST_WINDOWING_COMPUTATION):
            # display and record to a file all events found, checking with rolling idA from None to 4. Save nbEvent and total len
            eventTimeLineList = []
            eventList = getAllEvents(connection)
            file = open("outEvent" + str(windowT) + ".txt", "w")
            file.write("Event name\nnb event\ntotal duration")
            for eventName in eventList:
                for animal in range(0, 5):
                    idA = animal
                    if idA == 0:
                        idA = None
                    timeLine = EventTimeLineCached(connection, file, eventName, idA, minFrame=minT, maxFrame=maxT)
                    eventTimeLineList.append(timeLine)
                    file.write(timeLine.eventNameWithId + "\t" + str(len(timeLine.eventList)) + "\t" + str(
                        timeLine.getTotalLength()) + "\n")
            file.close()
        flushEventTimeLineCache()

    except:

        exc_type, exc_value, exc_traceback = sys.exc_info()
        lines = traceback.format_exception(exc_type, exc_value, exc_traceback)
        error = ''.join('!! ' + line for line in lines)

        t = TaskLogger(connection)
        t.addLog(error)
        flushEventTimeLineCache()

        raise FileProcessException()

def insertNightEventWithInputs(file, startNightInput, endNightInput):
    '''
    This function create night event
    '''

    connection = sqlite3.connect(file)

    nightTimeLine = EventTimeLine(connection, "night", None, None, None, None)

    flushNightEvents(connection)

    nightTimeLineFlushed = EventTimeLine(connection, "night", None, None, None, None)

    try:
        startNight = datetime.time(int(startNightInput.split(":")[0]), int(startNightInput.split(":")[1]),
                                   int(startNightInput.split(":")[2]))
    except ValueError:
        raise ValueError("Incorrect time format, should be hh:mm:ss")

    try:
        endNight = datetime.time(int(endNightInput.split(":")[0]), int(endNightInput.split(":")[1]),
                                 int(endNightInput.split(":")[2]))
    except ValueError:
        raise ValueError("Incorrect time format, should be hh:mm:ss")

    """
    Two cases: 
    - end night hour < start night hour means end night hour is the day after
    - end night hour > start night hour means the night is during the day: reverse cycle
    """

    if (endNight < startNight):
        cycle = "normal"
    else:
        cycle = "reverse"

    currentNight = Night(startHour=startNight, endHour=endNight, cycle=cycle)

    '''Beginning and end of the experiment'''
    startExperimentDate = getStartInDatetime(file)

    endExperimentDate = getEndInDatetime(file)

    currentDay = datetime.datetime.strftime(startExperimentDate, "%Y-%m-%d")
    currentDay = datetime.datetime(int(currentDay.split("-")[0]), int(currentDay.split("-")[1]),
                                   int(currentDay.split("-")[2]))
    previousDay = currentDay - datetime.timedelta(days=1)
    previousDay = datetime.datetime.strftime(previousDay, "%Y-%m-%d")

    currentStartNightDate = datetime.datetime.strptime("%s %s" % (previousDay, startNight), "%Y-%m-%d %H:%M:%S")

    lastFrame = getNumberOfFrames(file)

    currentNight.setStartEndDate(currentStartNightDate)

    while (True):
        if (currentNight.startDate > endExperimentDate):
            break

        tmpStartFrame = recoverFrame(file, str(currentNight.startDate))
        tmpEndFrame = recoverFrame(file, str(currentNight.endDate))

        if ((tmpStartFrame == 0) & (tmpEndFrame == 0)):
            if ((currentNight.startDate < startExperimentDate) & (currentNight.endDate > endExperimentDate)):
                tmpStartFrame = 1
                tmpEndFrame = lastFrame
                nightTimeLineFlushed.addEvent(Event(tmpStartFrame, tmpEndFrame))
                nightTimeLineFlushed.endRebuildEventTimeLine(connection, deleteExistingEvent=True)
            else:
                '''night outside the experiment'''
                pass
        else:
            if (tmpStartFrame == 0):
                tmpStartFrame = 1

            if (tmpEndFrame == 0):
                tmpEndFrame = lastFrame

            nightTimeLineFlushed.addEvent(Event(tmpStartFrame, tmpEndFrame))
            nightTimeLineFlushed.endRebuildEventTimeLine(connection, deleteExistingEvent=True)

        '''next day'''
        currentNight.nextDay()

def NightInputs():
    print("Construction of night events")
    night = input("Do you want to rebuild the night ? Yes (Y) or No (N) :")
    startNightInput = input("Time of the beginning of the night (hh:mm:ss):")
    endNightInput = input("Time of the end of the night (hh:mm:ss):")

    return night, startNightInput, endNightInput

def CreateNights(file, night, startNightInput, endNightInput):
    if (night == "Y") or (night == "Yes") or (night == "Ye") or (night == "Da") or (night == "Oui") or (night == "Si"):
        print(f"Code launched for file {file}")
        connection = sqlite3.connect(file)
        flushNightEvents(connection)
        insertNightEventWithInputs(file, startNightInput, endNightInput)
        print("*** NIGHTS CREATED ***")
    else:
        print("THE NIGHTS WILL NOT BE BUILD !!!!")

# --------------------------------------------------------------------------------------------------------
# End of rebuild
# --------------------------------------------------------------------------------------------------------
# --------------------------------------------------------------------------------------------------------
# Start of export
# --------------------------------------------------------------------------------------------------------
def computeBehaviorsData(behavior, show=False):
    """ Computes the details (Mean, Nb, Std, CI95...) of behaviors."""

    name = behavior.eventName
    # nameAndIds = behavior.eventNameWithId
    idA = behavior.idA
    idB = behavior.idB
    idC = behavior.idC
    idD = behavior.idD
    totalLength = behavior.getTotalLength()
    meanLength = behavior.getMeanEventLength()
    numberOfEvents = behavior.getNumberOfEvent()
    stdLength = behavior.getStandardDeviationEventLength()

    """    if behavior.getMedianEventLength() is not None:
        medianLength = behavior.getMedianEventLength()
    else:
        medianLength = None"""

    medianLength = None

    CI95 = [None]*2
    if meanLength is not None:
        CI = 1.96 * behavior.getStandardDeviationEventLength() / np.math.sqrt(behavior.getNumberOfEvent())
        CI95[0] = meanLength - CI
        CI95[1] = meanLength + CI
    else:
        meanLength = None

    returnedBehaviors = {
        "name": name,
        "idA": idA,
        "idB": idB,
        "idC": idC,
        "idD": idD,
        "totalLength": totalLength,
        "meanLength": meanLength,
        "medianLength": medianLength,
        "numberOfEvents": numberOfEvents,
        "stdLength": stdLength,
        "CI95_low": CI95[0],
        "CI95_up": CI95[1]
    }

    # print(f"{nameAndIds}")
    # print(f"   -TotalLength: {totalLength} frames.")
    # print(f"   -MeanEventLength: {meanLength}")
    # print(f"   -MedianEventLength: {medianLength}")
    # print(f"   -NumberOfEvent: {numberOfEvents}")
    # print(f"   -StandardDeviationEventLength: {stdLength}")
    # print(f"   -The confidence interval (95%) is [{CI95[0]},{CI95[1]}].")

    # if show:
        # beh.plotEventDurationDistributionHist(nbBin=30, title="Timebin #"+str(bin) + " / " + beh.eventNameWithId)
        # beh.plotEventDurationDistributionBar(title="Timebin #"+str(bin) + " / " + beh.eventNameWithId)

    # TODO: Show the histogram of events, with a 'show' parameter, with one subplot per behavior

    return returnedBehaviors

def RebuildInputs():

    # Questions for the rebuild of the databases
    buildEvents = input("Do you want to rebuild the Events ?")
    # Questions for the creation of the csv files
    timeBinsDuration = int(input("Enter the TIMEBIN for ALL the files (1min =  1800 frames / 1h = 108000 frames): "))
    useNights = input("Do you want to use the Nights from the .sqlite files to computes the data ? ('Yes'/'No'): ")

    return buildEvents, timeBinsDuration, useNights

def Rebuild(file, files, buildEvents):

    global timeBinsDuration
    global useNights

    print("Processing file, please wait...")

    chronoFullBatch = Chronometer("Full batch")

    fileCount = 0  # File Counter
    if files != None:
        if fileCount == 0:  # First file
            try:
                fileCount += 1  # Increment file Counter
                if buildEvents == "Yes" or buildEvents == 'Y' or buildEvents == "yes":
                    process(file)
            except FileProcessException:
                print("STOP PROCESSING FILE " + file, file=sys.stderr)
        else:  # For other files than the first one
            try:
                process(file)
            except FileProcessException:
                print("STOP PROCESSING FILE " + file, file=sys.stderr)

def Export(count:int, file, files, timeBinsDuration, useNights):

    filenames = [os.path.basename(files[x]) for x in range(0, len(files))]

    ### DEFINE CONSTANTS ###
    start = {}
    stop = {}

    # Create a Global Dataframe with all data:
    dfGlobal = pd.DataFrame()

    fileName = filenames[count]

    # Check that 'filename' and the current file are the same
    # TODO CAN BE IMPROVED by extracting from the current file

    connection = sqlite3.connect(file)  # connect to database
    animalPool = AnimalPool()  # create an animalPool, which basically contains your animals
    animalPool.loadAnimals(connection)  # load infos about the animals

    animalNumber = animalPool.getNbAnimals()

    if useNights.lower() == "yes" or useNights.lower() == "y":
        NightFrames = animalPool.getNightStartStop()

        # TODO find number of night and create bins for each

        nbTimebin = []

        startFrame = NightFrames[0][0]
        stopFrame = NightFrames[0][1]

        nbTimebin.append(int((stopFrame - startFrame) / timeBinsDuration) + 1)

        nbTimebins = int((stopFrame - startFrame) / timeBinsDuration) + 1

        for filename in filenames:
            start[filename] = startFrame
            stop[filename] = stopFrame

    if useNights.lower() == "no" or useNights.lower() == "no":
        print("You should use the nights")

    # TODO Flush ALL the events !!

    # Different events Depending on the number of animals in the Database:
    if animalNumber >= 1:
        behavioralEventsForOneAnimal = ["Move", "Move isolated", "Rearing", "Rear isolated",
                                        "Stop isolated", "WallJump", "SAP", "Huddling", "WaterPoint", "Distance"]
        # print("The behaviors extracted are:\n", behavioralEventsForOneAnimal)
    if animalNumber >= 2:
        behavioralEventsForTwoAnimals = ["Contact", "Oral-oral Contact", "Oral-genital Contact",
                                         "Side by side Contact",
                                         "Side by side Contact, opposite way", "Social approach", "Social escape",
                                         "Approach contact", "Approach rear", "Break contact", "Get away",
                                         "FollowZone Isolated",
                                         "Train2", "Group2", "Move in contact", "Rear in contact"]
        # print("and:\n", behavioralEventsForTwoAnimals)
    if animalNumber >= 3:
        behavioralEventsForThreeAnimals = ["Group3", "Group 3 break", "Group 3 make"]
        # print("and:\n", behavioralEventsForThreeAnimals)
    if animalNumber >= 4:
        behavioralEventsForFourAnimals = ["Group4", "Group 4 break", "Group 4 make", "Nest3", "Nest4"]
        # print("and:", behavioralEventsForFourAnimals)

    # Export the Infos (Mean, Std, ...) of the Behaviors for each timebin:
    dicoOfBehInfos = {
        "Filename": None,
        "Date": None,
        "Cage": None,
        "Injection": None,
        "Night-Phase": None,
        "Bin": None,
        "start_frame": None,
        "stop_frame": None,
        "name": None,
        "idA": None,
        "idB": None,
        "idC": None,
        "idD": None,
        "RFidA": None,
        "RFidB": None,
        "RFidC": None,
        "RFidD": None,
        "GenoA": None,
        "GenoB": None,
        "GenoC": None,
        "GenoD": None,
        "totalLength": None,
        "meanLength": None,
        "medianLength": None,
        "numberOfEvents": None,
        "stdLength": None,
        "CI95_low": None,
        "CI95_up": None
    }

    dfOfBehInfos = pd.DataFrame()  # Dataframe with the details of the previous dict

    allBehaviorsInfo = {}

    # TODO one for loop on Nights and One for loop for the times bins inside the night
    night_count = 1
    for night in NightFrames:
        bin = 1
        bin_progress = tqdm(range(1, nbTimebins + 1), desc=f"Night {night_count} - Bins", position=0, leave=True)
        for z in bin_progress:
            startBin = (start[fileName] + (z - 1) * timeBinsDuration) + (night_count - 1) * (108000 * 24)
            stopBin = startBin + timeBinsDuration
            bin_progress.set_postfix(Bin=f"{bin}/{nbTimebins}")
            animalPool.loadDetection(start=startBin, end=stopBin)  # load the detection for the different bins

            # Update the dictionary with Start; Stop; and bin info:
            dicoOfBehInfos["start_frame"] = startBin
            dicoOfBehInfos["stop_frame"] = stopBin
            dicoOfBehInfos["Bin"] = bin
            dicoOfBehInfos["Night-Phase"] = night_count

            # Update the dictionary with Filename; Date; Cage; and Session info
            # Example file name "191104_Magel2_Cage4_LMT3_4mo - Copy.sqlite"
            dicoOfBehInfos["Filename"] = fileName[:-7]  # Remove the " - Copy.sqlite" part
            date, Xp, cage, Injection = fileName[:-7].split("_")  # Splits the name at '_'
            dicoOfBehInfos["Date"] = date
            dicoOfBehInfos["Cage"] = cage
            dicoOfBehInfos["Injection"] = Injection

            #### For 1+ ANIMAL ####
            if animalNumber >= 1:
                for behavior in behavioralEventsForOneAnimal:
                    for a in animalPool.getAnimalDictionnary():
                        if behavior == "Distance":
                            dist_temp = animalPool.getAnimalWithId(a).getDistance(startBin, stopBin)
                            dicoOfBehInfos.update({"name": "Distance", "totalLength": dist_temp,
                                                   "RFidA": animalPool.getAnimalWithId(a).RFID,
                                                   "GenoA": animalPool.getAnimalWithId(a).genotype,
                                                   "idA": a})
                        else:

                            eventTimeLine = EventTimeLine(connection, behavior, idA=a, minFrame=startBin,
                                                          maxFrame=stopBin)

                            behavioralDataOne = computeBehaviorsData(eventTimeLine)  # Computes the Stats.
                            dicoOfBehInfos.update(behavioralDataOne)  # ADD THE CALCULATIONS FROM THE EVENTIMELINE
                            dicoOfBehInfos.update({"RFidA": animalPool.getAnimalWithId(a).RFID,
                                                   "GenoA": animalPool.getAnimalWithId(a).genotype})

                        # Creates a dataFrame with the behavioral Infos from the dictionary:
                        index = [0]
                        dfOfBehInfosTemp = pd.DataFrame(dicoOfBehInfos, index=index)
                        dfOfBehInfos = dfOfBehInfos.append(dfOfBehInfosTemp, ignore_index=True)

                    # TODO for each Timebin, Plot the timeline of the behaviors ?
                    # plotMultipleTimeLine(behavioralList, title=behavior+" / timebin #"+str(bin), minValue=start)

            #### FOR 2+ ANIMALS ####
            if animalNumber >= 2:
                for behavior in behavioralEventsForTwoAnimals:
                    # print("**** ", behavior, " ****")
                    behavioralList2 = []

                    for a in animalPool.getAnimalDictionnary():
                        if behavior == "Move in contact" or behavior == "Rear in contact":  # Just idA in those behaviors
                            eventTimeLine = EventTimeLine(connection, behavior, idA=a, minFrame=startBin,
                                                          maxFrame=stopBin)
                            continue
                        for b in animalPool.getAnimalDictionnary():
                            if a == b:
                                continue
                            eventTimeLine = EventTimeLine(connection, behavior, idA=a, idB=b,
                                                          minFrame=startBin, maxFrame=stopBin)

                            behavioralDataTwo = computeBehaviorsData(eventTimeLine)  # Computes the Stats.
                            dicoOfBehInfos.update(behavioralDataTwo)  # ADD THE CALCULATIONS FROM THE EVENTIMELINE
                            dicoOfBehInfos.update({"RFidA": animalPool.getAnimalWithId(a).RFID,
                                                   "RFidB": animalPool.getAnimalWithId(b).RFID,
                                                   "GenoA": animalPool.getAnimalWithId(a).genotype,
                                                   "GenoB": animalPool.getAnimalWithId(b).genotype})

                            # Creates a dataFrame with the behavioral Infos from the dictionary:
                            index = [0]
                            dfOfBehInfosTemp = pd.DataFrame(dicoOfBehInfos, index=index)
                            dfOfBehInfos = dfOfBehInfos.append(dfOfBehInfosTemp, ignore_index=True)

            #### FOR 3+ ANIMALS ####
            if animalNumber >= 3:
                for behavior in behavioralEventsForThreeAnimals:
                    for a in animalPool.getAnimalDictionnary():
                        # There is ONLY ONE animal making or braking a group of 3 or 4 mice
                        if behavior == "Group 3 make" or behavior == "Group 3 break":
                            eventTimeLine = EventTimeLine(connection, behavior, idA=a, minFrame=startBin,
                                                          maxFrame=stopBin)
                            continue
                        for b in animalPool.getAnimalDictionnary():
                            if a == b:
                                continue
                            for c in animalPool.getAnimalDictionnary():
                                if a == c or b == c:
                                    continue
                                eventTimeLine = EventTimeLine(connection, behavior, idA=a, idB=b, idC=c,
                                                              minFrame=startBin, maxFrame=stopBin)

                                behavioralDataThree = computeBehaviorsData(eventTimeLine)  # Computes the Stats.
                                dicoOfBehInfos.update(
                                    behavioralDataThree)  # ADD THE CALCULATIONS FROM THE EVENTIMELINE
                                dicoOfBehInfos.update({"RFidA": animalPool.getAnimalWithId(a).RFID,
                                                       "RFidB": animalPool.getAnimalWithId(b).RFID,
                                                       "RFidC": animalPool.getAnimalWithId(c).RFID,
                                                       "GenoA": animalPool.getAnimalWithId(a).genotype,
                                                       "GenoB": animalPool.getAnimalWithId(b).genotype,
                                                       "GenoC": animalPool.getAnimalWithId(c).genotype})

                                # Creates a dataFrame with the behavioral Infos from the dictionary:
                                index = [0]
                                dfOfBehInfosTemp = pd.DataFrame(dicoOfBehInfos, index=index)
                                dfOfBehInfos = dfOfBehInfos.append(dfOfBehInfosTemp, ignore_index=True)

            #### FOR 4 ANIMALS ####
            if animalNumber >= 4:
                for behavior in behavioralEventsForFourAnimals:
                    for a in animalPool.getAnimalDictionnary():
                        # There is ONLY ONE animal making or braking a group of 3 or 4 mice
                        if behavior == "Group 4 make" or behavior == "Group 4 break":
                            eventTimeLine = EventTimeLine(connection, behavior, idA=a,
                                                          minFrame=startBin, maxFrame=stopBin)
                            continue
                        for b in animalPool.getAnimalDictionnary():
                            if a == b:
                                continue
                            for c in animalPool.getAnimalDictionnary():
                                if a == c or b == c:
                                    continue
                                # There is ONLY THREE animals making a Nest3
                                if behavior == "Nest3":
                                    eventTimeLine = EventTimeLine(connection, behavior, idA=a, idB=b, idC=c,
                                                                  minFrame=startBin, maxFrame=stopBin)
                                    continue
                                for d in animalPool.getAnimalDictionnary():
                                    if a == d or b == d or c == d:
                                        continue
                                    eventTimeLine = EventTimeLine(connection, behavior, idA=a, idB=b, idC=c, idD=d,
                                                                  minFrame=startBin, maxFrame=stopBin)

                                    behavioralDataFour = computeBehaviorsData(eventTimeLine)
                                    dicoOfBehInfos.update(behavioralDataFour)
                                    dicoOfBehInfos.update({"RFidA": animalPool.getAnimalWithId(a).RFID,
                                                           "RFidB": animalPool.getAnimalWithId(b).RFID,
                                                           "RFidC": animalPool.getAnimalWithId(c).RFID,
                                                           "RFidD": animalPool.getAnimalWithId(d).RFID,
                                                           "GenoA": animalPool.getAnimalWithId(a).genotype,
                                                           "GenoB": animalPool.getAnimalWithId(b).genotype,
                                                           "GenoC": animalPool.getAnimalWithId(c).genotype,
                                                           "GenoD": animalPool.getAnimalWithId(d).genotype})

                                    # Creates a dataFrame with the behavioral Infos from the dictionary:
                                    index = [0]
                                    dfOfBehInfosTemp = pd.DataFrame(dicoOfBehInfos, index=index)
                                    dfOfBehInfos = dfOfBehInfos.append(dfOfBehInfosTemp, ignore_index=True)

            bin += 1
        night_count += 1
    # count += 1  # TODO REPLACE THIS COUNT BY A FOR LOOP ON FILES

    dfOfBehInfos.to_csv(f"{fileName}.csv")  # Export the current dataframe into a .csv
    print(f"{fileName}.csv File Created !")
    print("######################## Close Connection with Database ##########################")
    connection.close()

# --------------------------------------------------------------------------------------------------------
# End of export
# --------------------------------------------------------------------------------------------------------

# # Select databases to rebuild and create csv
#
# files = getFilesToProcess()
#
# # Questions for the rebuild of the databases
#
# buildEvents = input("Do you want to rebuild the Events ?")
#
# # Questions for the creation of the csv files
#
# timeBinsDuration = int(input("Enter the TIMEBIN for ALL the files (1min =  1800 frames / 1h = 108000 frames): "))
# useNights = input("Do you want to use the Nights from the .sqlite files to computes the data ? ('Yes'/'No'): ")
#
# count = 0
# print(files)
#
# for file in files:
#     rebuild(file, files, buildEvents, night, startNightInput, endNightInput)
#     Export(count, file, files, timeBinsDuration, useNights)
#     count += 1




'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.EventTimeLineCache import EventTimeLineCached

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "Group 3 make" )
    deleteEventTimeLineInBase(connection, "Group 3 break" )


def reBuildEvent( connection, file, tmin=None, tmax=None, pool = None  ):
    

    pool = AnimalPool( )
    pool.loadAnimals( connection )
    #pool.loadDetection( start = tmin, end = tmax )
    
    
    ''' load contact matrix '''
    contact = {}
    group3In = {}
    group3Out = {}
    
    for animal in range( 1 , 5 ):
        contact[animal] = EventTimeLineCached( connection, file, "Contact", animal, minFrame=tmin, maxFrame=tmax )
        group3In[animal] = EventTimeLine( connection, "Group 3 make", animal, loadEvent=False )
        group3Out[animal] = EventTimeLine( connection, "Group 3 break", animal, loadEvent=False )
        

    ''' process group '''
    
    for animal in range( 1 , 5 ):
        for idAnimalB in range( 1 , 5 ):
            for idAnimalC in range( 1 , 5 ):

                ''' check impossible combination (avoid 2 animals with same id) '''
                test = {}
                test[animal] = True
                test[idAnimalB] = True
                test[idAnimalC] = True
                
                if not len ( test.keys() ) == 3:
                    continue                

                ''' process group '''

                group3 = EventTimeLineCached( connection, file, "Group3", animal, idAnimalB, idAnimalC, minFrame=tmin, maxFrame=tmax )
                
                eventList = group3.getEventList()
                
                for event in eventList:
                    
                    t = event.startFrame-1

                    if ( not contact[animal].hasEvent( t ) ):                        
                        group3In[animal].addPunctualEvent( t )

                    if ( not contact[idAnimalB].hasEvent( t ) ):                        
                        group3In[idAnimalB].addPunctualEvent( t )

                    if ( not contact[idAnimalC].hasEvent( t ) ):                        
                        group3In[idAnimalC].addPunctualEvent( t )
                        
                    t = event.endFrame+1

                    if ( not contact[animal].hasEvent( t ) ):                        
                        group3Out[animal].addPunctualEvent( t )

                    if ( not contact[idAnimalB].hasEvent( t ) ):                        
                        group3Out[idAnimalB].addPunctualEvent( t )

                    if ( not contact[idAnimalC].hasEvent( t ) ):                        
                        group3Out[idAnimalC].addPunctualEvent( t )

                
    ''' save all '''
    for idAnimal in range( 1 , 5 ):
        
        group3In[idAnimal].endRebuildEventTimeLine(connection)
        group3Out[idAnimal].endRebuildEventTimeLine(connection)

        
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Group3 Make Break" , tmin=tmin, tmax=tmax )

                                                   
    # print( "Rebuild event finished." )
        
    


'''
Created on 2. February 2021

@author: Elodie
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
#from affine import Affine
import matplotlib.pyplot as plt
import matplotlib.lines as mlines

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "SniffRight" )
    deleteEventTimeLineInBase(connection, "SniffLeft" )
    

def reBuildEvent( connection, exp, phase, objectPosition, radiusObjects, objectDic, tmin=None, tmax=None, pool = None ):
    deleteEventTimeLineInBase(connection, "SniffRight" )
    deleteEventTimeLineInBase(connection, "SniffLeft")
    deleteEventTimeLineInBase(connection, "upRight")
    deleteEventTimeLineInBase(connection, "upLeft")
    ''' use the pool provided or create it'''
    if ( pool == None ):
        pool = AnimalPool( )
        pool.loadAnimals( connection )
        pool.loadDetection( start = tmin, end = tmax )
        pool.filterDetectionByInstantSpeed(0, 70)
    '''
    Event SniffRight
    - the animal's nose is in a zone 3 cm around the object at the right
    
    Event SniffLeft
    - the animal's nose is in a zone 3 cm around the object at the left
    
    Event upRight
    - the animal is on the object located on the right
    
    Event upLeft
    - the animal is on the object located on the left
    '''
    
    for animal in pool.animalDictionnary.keys():
        # print(pool.animalDictionnary[animal])
        
        eventNameSniffRight = "SniffRight"
        # print ( "A is around the object located on the right")
        # print ( eventNameSniffRight )
        
        eventNameSniffLeft = "SniffLeft"
        # print ( "A is around object located on the left")
        # print ( eventNameSniffLeft )

        eventNameUpRight = "UpRight"
        # print("A is on the object located on the right")
        # print(eventNameUpRight)

        eventNameUpLeft = "UpLeft"
        # print("A is on object located on the left")
        # print(eventNameUpLeft)
                
        sniffRightTimeLine = EventTimeLine( None, eventNameSniffRight , animal , None , None , None , loadEvent=False )
        sniffLeftTimeLine = EventTimeLine( None, eventNameSniffLeft , animal , None , None , None , loadEvent=False )
        upRightTimeLine = EventTimeLine(None, eventNameUpRight, animal, None, None, None, loadEvent=False)
        upLeftTimeLine = EventTimeLine(None, eventNameUpLeft, animal, None, None, None, loadEvent=False)

        resultSniffRight = {}
        resultSniffLeft = {}
        resultUpRight = {}
        resultUpLeft = {}
        
        animalA = pool.animalDictionnary[animal]
        setup = int(animalA.setup)
        objectLeft = objectDic[setup][exp][phase][0]
        objectRight = objectDic[setup][exp][phase][1]
        #print ( animalA )
        dicA = animalA.detectionDictionnary

        for t in dicA.keys():
            #print(t)
            distanceNoseLeft = animalA.getDistanceNoseToPoint(t=t, xPoint=objectPosition[setup]['left'][0], yPoint=-objectPosition[setup]['left'][1])
            distanceMassLeft = animalA.getDistanceToPoint(t=t, xPoint=objectPosition[setup]['left'][0], yPoint=-objectPosition[setup]['left'][1])
            #print('distance left: ', distanceNoseLeft)
            if distanceNoseLeft == None:
                print('no nose detected for frame ', t)

            else:
                if distanceNoseLeft <= radiusObjects[objectLeft] + 3 / scaleFactor:
                    # check if the animal is on the object:
                    if distanceMassLeft <= radiusObjects[objectLeft]:
                        resultUpLeft[t] = True
                    else:
                        resultSniffLeft[t] = True

            distanceNoseRight = animalA.getDistanceNoseToPoint(t=t, xPoint=objectPosition[setup]['right'][0], yPoint=-objectPosition[setup]['right'][1])
            distanceMassRight = animalA.getDistanceToPoint(t=t, xPoint=objectPosition[setup]['right'][0], yPoint=-objectPosition[setup]['right'][1])
            #print('distance right: ', distanceNoseRight)
            if distanceNoseRight == None:
                print('no nose detected for frame ', t)

            else:
                if distanceNoseRight <= radiusObjects[objectRight] + 3 / scaleFactor:
                    # check if the animal is on the object:
                    if distanceMassRight <= radiusObjects[objectRight]:
                        resultUpRight[t] = True
                    else:
                        resultSniffRight[t] = True

        sniffRightTimeLine.reBuildWithDictionnary( resultSniffRight )
        sniffRightTimeLine.endRebuildEventTimeLine(connection)

        sniffLeftTimeLine.reBuildWithDictionnary(resultSniffLeft)
        sniffLeftTimeLine.endRebuildEventTimeLine(connection)

        upRightTimeLine.reBuildWithDictionnary(resultUpRight)
        upRightTimeLine.endRebuildEventTimeLine(connection)

        upLeftTimeLine.reBuildWithDictionnary(resultUpLeft)
        upLeftTimeLine.endRebuildEventTimeLine(connection)

    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Sniff Right" , tmin=tmin, tmax=tmax )
    t.addLog( "Build Event Sniff Left" , tmin=tmin, tmax=tmax )
    t.addLog("Build Event Up Right", tmin=tmin, tmax=tmax)
    t.addLog("Build Event Up Left", tmin=tmin, tmax=tmax)

    # print( "Rebuild event finished." )
        
        
        
        
        
        
        
    


'''
Created on 6 sept. 2017

@author: Fab
'''


def getNumberOfIndexOfDatabase( connection ):
    c = connection.cursor()
            
    query = "select type, name, tbl_name, sql FROM sqlite_master WHERE type='index'";
    c.execute( query )
    rows = c.fetchall()
    nbIndex = len( rows )
    # print( "Number of index in lmtanalysis: ", nbIndex )
    return nbIndex

def executeLog( cursor, query):
    try:
        # print( query )
        cursor.execute( query )
    except Exception as e:
        print (e)
    

def buildDataBaseIndex( connection , force=False):

    # print( "Creating lmtanalysis indexes...")

    c = connection.cursor()            

    '''
    if ( force == False ):
        if( getNumberOfIndexOfDatabase( connection ) > 0 ):
            print( "Index already exists (maybe not all)... no re-indexing. Use buildDataBaseIndex force parameter to force index build.")
            return
    '''
     
    executeLog( c , "CREATE INDEX `animalIndex` ON `ANIMAL` (`ID` );" )     

    executeLog( c , "CREATE INDEX `detectionIndex` ON `DETECTION` (`ID` ASC,`FRAMENUMBER` ASC);" )     
        
    executeLog( c , "CREATE INDEX `detetIdIndex` ON `DETECTION` (`ID` ASC);" )     
        
    executeLog( c , "CREATE INDEX `detframenumberIndex` ON `DETECTION` (`FRAMENUMBER` ASC);" )     
        
    executeLog( c , "CREATE INDEX `eventEndFrameIndex` ON `EVENT` (`ENDFRAME` ASC);" )     
        
    executeLog( c , "CREATE INDEX `eventIndex` ON `EVENT` (`ID` ASC,`STARTFRAME` ASC,`ENDFRAME` ASC);" )     

    executeLog( c , "CREATE INDEX `eventStartFrameIndex` ON `EVENT` (`STARTFRAME` ASC);" )     
        
    executeLog( c , "CREATE INDEX `eventstartendIndex` ON `EVENT` (`STARTFRAME` ASC,`ENDFRAME` ASC);" )     
        
    executeLog( c , "CREATE INDEX `indexeventidIndex` ON `EVENT` (`ID` ASC);" )     
    
    executeLog( c , "CREATE INDEX 'detectionFastLoadXYIndex' ON 'DETECTION' ('ANIMALID' ,'FRAMENUMBER' ASC,'MASS_X' ,'MASS_Y' );" )
    
    
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build DataBase Index" )

        





'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
#from affine import Affine
import matplotlib.pyplot as plt
import matplotlib.lines as mlines
import sys
import matplotlib.pyplot as plt

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "WallJump" )

def reBuildEvent( connection, file, tmin=None, tmax=None, pool = None, showGraph = False ): 
    
    ''' use the pool provided or create it'''
    if ( pool == None ):
        pool = AnimalPool( )
        pool.loadAnimals( connection )
        pool.loadDetection( start = tmin, end = tmax )
    
    centerX = 512/2
    centerY = 424/2
    
    if ( showGraph ):
        plt.figure( 1 )
    
    
    
    for animal in pool.animalDictionnary.keys():
        # print(pool.animalDictionnary[animal])
        
        if ( showGraph ):    
            plt.subplot(2,2, animal )
        
        eventName = "WallJump"
        # print ( "A is jumping against the wall")
        # print ( eventName )
                
        JumpWallTimeLine = EventTimeLine( None, eventName , animal , None , None , None , loadEvent=False )
        
        result={}
        
        animalA = pool.animalDictionnary[animal]
        #print ( animalA )
        dicA = animalA.detectionDictionnary

        amplitude = 28
        window = 6
        threshold = 100
        angleTolerance = math.pi/8
        maxMissingDetection = 3
            
        for t in dicA.keys():

            #check if data is available
            numberOfDataNotAvailable = 0
            for tt in range( t, t+window+1 ):
                if ( not ( tt in dicA ) ):
                    numberOfDataNotAvailable +=1
                
            if ( numberOfDataNotAvailable > maxMissingDetection ):
                continue            

            jumpFound= False
            totalDistance = 0

            dis1 = math.hypot( dicA[t].massX - centerX, dicA[t].massY - centerY )
            dis2 = math.hypot( dicA[t].frontX - centerX, dicA[t].frontY - centerY )
            dis3 = math.hypot( dicA[t].backX - centerX, dicA[t].backY - centerY )

            distanceStart = min( dis1, dis2, dis3 )

            ''' check orientation to the external of the cage '''
            orientationValid = True
            for tt in range( t, t+window+1 ):
                
                if ( tt in dicA ):
                
                    directionAnimalA = dicA[tt].getDirection()
                    directionCenterAnimal = math.atan2( centerY - dicA[tt].massY, centerX - dicA[tt].massX )
                    
                    #same direction
                    angleDif1 = math.atan2( math.sin(directionCenterAnimal-directionAnimalA), math.cos(directionCenterAnimal-directionAnimalA) )
                    angleDif1 = math.fabs( angleDif1 ) 
                    
                    #opposite direction
        
                    angleDif2 = math.atan2( math.sin(directionCenterAnimal+math.pi-directionAnimalA), math.cos(directionCenterAnimal+math.pi-directionAnimalA) )
                    angleDif2 = math.fabs( angleDif2 ) 
                    
                    angleDif = min( angleDif1 , angleDif2 )
                    
                    if ( angleDif > angleTolerance ):
                        orientationValid = False
                        break
                
            if not orientationValid:
                continue

            ''' check if animal trajectory match jump model '''

            xList = []
            yList = []

            for tt in range( t, t+window+1 ): 
        
                if ( tt in dicA ):                    
                    
                    dis1 = math.hypot( dicA[tt].massX - centerX, dicA[tt].massY - centerY )
                    dis2 = math.hypot( dicA[tt].frontX - centerX, dicA[tt].frontY - centerY )
                    dis3 = math.hypot( dicA[tt].backX - centerX, dicA[tt].backY - centerY )
                    
                    distanceToCenter = min( dis1, dis2, dis3 )
                    
                    #distanceToCenter = math.hypot( dicA[tt].massX - centerX, dicA[tt].massY - centerY )
                    
                    distanceToStart = distanceToCenter - distanceStart
                    idealSin = math.sin( 2*math.pi * ( tt-t - math.pi/2 ) / window ) * amplitude/2 + amplitude/2 
                    #idealSin = math.sin( 2*math.pi * ( tt-t - math.pi/2 ) / window ) * 15
                    distance = math.pow( idealSin - distanceToStart , 2 )
    
                    totalDistance+= distance
                    xList.append( tt-t )
                    yList.append( distanceToCenter - distanceStart )                
            
            totalDistance/=window
                        
            if ( totalDistance < threshold ):
                jumpFound = True
                
                if ( showGraph ):
                    plt.plot( xList, yList, linestyle='-', linewidth=1 )            
                
                #print( "jump found. dist={} and t={}".format( totalDistance,t+window/2 ) )
                                       
            if ( jumpFound ):
                ''' build the event on a thinner range to avoid event overlap and automatic continuity '''
                for tt in range( t+2, t+window-2 ): 
                    result[tt] = True
                
        xList = []
        yList = [] 
        
        if ( showGraph ):
            for tt in range( t, t+window+1 ): 
                idealSin = math.sin( 2*math.pi * ( tt-t - math.pi/2 ) / window ) * amplitude/2 + amplitude/2
                xList.append( tt-t )
                yList.append( idealSin )
            plt.plot( xList, yList, linestyle='--', linewidth=4 )
        
        JumpWallTimeLine.reBuildWithDictionnary( result )            
        JumpWallTimeLine.endRebuildEventTimeLine(connection)
        
    if ( showGraph ):
        plt.show()
    
        
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Wall Jump" , tmin=tmin, tmax=tmax )

     
    # print( "Rebuild event finished." )
        
        
        
        
        
        
        
    


'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.EventTimeLineCache import EventTimeLineCached


def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "Approach contact" )

def reBuildEvent( connection, file, tmin=None, tmax=None , pool = None ): 
    
    '''
    This event represents the approaches that is just before a contact (longer than 3 frames); if the speeds of the two animals
    involved in the contact are not different enough, the animal approaching cannot be defined and therefore the event is not computed 
    for this contact.
    This event is a reduction of distance between two animals, one going more quickly than the other, until they get in contact.
    This event ends at the frame just before the contact and lasts the whole time of approach.
    '''
    
    #pool = AnimalPool( )
    #pool.loadAnimals( connection )
    #pool.loadDetection( start = tmin, end = tmax )
    if ( pool == None ):
        pool = AnimalPool( )
        pool.loadAnimals( connection )
        pool.loadDetection( start = tmin, end = tmax , lightLoad=True )
    
    eventName = "Approach contact"        
    # print ( eventName )
    # print('Computing ', eventName)
            
    #upload the timelines of interest
    contactDico = {}
    approachDico = {}
    for animal in range( 1 , pool.getNbAnimals()+1 ):
        for idAnimalB in range( 1 , pool.getNbAnimals()+1 ):
            if ( animal == idAnimalB ):
                continue
            #charge the contact timelines as a dictionary of timelines
            contactDico[animal, idAnimalB] = EventTimeLineCached( connection, file, "Contact", animal, idAnimalB, minFrame=tmin, maxFrame=tmax )
            #charge the approach timelines as a dictionary of timelines
            approachDico[animal, idAnimalB] = EventTimeLineCached( connection, file, "Approach", animal, idAnimalB, minFrame=tmin, maxFrame=tmax ) #fait une matrice de toutes les approches à deux possibles
    
    #initiate the timeline of approach before a contact
    appContactTimeLineDico = {}
    appContactTimeLineDicoDico = {}
    for animal in range( 1 , pool.getNbAnimals()+1 ):
        for idAnimalB in range( 1 , pool.getNbAnimals()+1 ):
            if ( animal == idAnimalB ):
                continue
            
            appContactTimeLineDico[ animal, idAnimalB ] = EventTimeLine( None, eventName , animal , idAnimalB , None , None , loadEvent=False )
            appContactTimeLineDicoDico[ animal, idAnimalB ] = {}
    
    #upload the dictionary for each approach timeline
    approachDicoDico = {}
    for animal in range( 1 , pool.getNbAnimals()+1 ):
        for idAnimalB in range( 1 , pool.getNbAnimals()+1 ):
            if( animal == idAnimalB ):
                continue
            
            #upload the dictionary for each approach timeline:
            approachDicoDico[animal, idAnimalB] = approachDico[animal, idAnimalB].getDictionary()
            
    #check who approached who before the contact
    for animal in range( 1 , pool.getNbAnimals()+1 ):
        for idAnimalB in range( 1 , pool.getNbAnimals()+1 ):
            if( animal == idAnimalB ):
                continue
            
            #clean the contact timeline to keep only events longer than 3 frames
            contactDico[animal, idAnimalB].mergeCloseEvents(numberOfFrameBetweenEvent=1)
            contactDico[animal, idAnimalB].removeEventsBelowLength(maxLen=3)
            
            
            for contactEvent in contactDico[animal, idAnimalB].eventList:
                frameBeforeContact = contactEvent.startFrame - 1
                #if the frame before the contact is within the approach timeline of A to B, then add the corresponding approach event to the approach contact timeline
                if frameBeforeContact in approachDicoDico[animal, idAnimalB]:
                    #print('A approaches B')
                    approachEvent = approachDico[animal, idAnimalB].getEventAt( frameBeforeContact )
                    for t in range (approachEvent.startFrame, frameBeforeContact + 1):
                        appContactTimeLineDicoDico[animal, idAnimalB][t] = True
                
                #if the frame before the contact is within the approach timeline of B to A, then add the corresponding approach event to the approach contact timeline
                elif frameBeforeContact in approachDicoDico[idAnimalB, animal]:
                    #print('B approaches A')
                    approachEvent = approachDico[idAnimalB, animal].getEventAt( frameBeforeContact )
                    for t in range (approachEvent.startFrame, frameBeforeContact + 1):
                        appContactTimeLineDicoDico[idAnimalB, animal][t] = True
                
                #if the frame before the contact is not within any approach timeline, then go to the next contact event
                else:
                    #print('Not possible to decide who is approaching who.')
                    continue
                
 
    for animal in range( 1 , pool.getNbAnimals()+1 ):
        for idAnimalB in range( 1 , pool.getNbAnimals()+1 ):
            if( animal == idAnimalB ):
                continue
            appContactTimeLineDico[animal, idAnimalB].reBuildWithDictionnary( appContactTimeLineDicoDico[animal, idAnimalB] )
            appContactTimeLineDico[animal, idAnimalB].endRebuildEventTimeLine( connection )
    
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Approach Contact" , tmin=tmin, tmax=tmax )
                       
    # print( "Rebuild event finished." )
        
    


'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.EventTimeLineCache import EventTimeLineCached

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "Approach rear" )

def reBuildEvent( connection, file, tmin=None, tmax=None , pool = None ):
    ''' 
    Animal A is approaching the animal B, that is in a reared posture at the end of the approach of animal A.
    Social approaches are considered, meaning that the two animals are within 2 body lengths of one another.
    Rearings are considered "in contact" given that the computation is done on the last second of an approach.
    ''' 
    
    #pool = AnimalPool( )
    #pool.loadAnimals( connection )
    #pool.loadDetection( start = tmin, end = tmax )
    
    if ( pool == None ):
        pool = AnimalPool( )
        pool.loadAnimals( connection )
        pool.loadDetection( start = tmin, end = tmax , lightLoad=True )
        
    
    rearDico = {}
    approachDico = {}
    dicoEventRearAnimal = {}
    for animal in range( 1 , 5 ):

        rearDico[animal] = EventTimeLine( connection, "Rear in contact", animal, None, minFrame=tmin, maxFrame=tmax )
        dicoEventRearAnimal[animal] = rearDico[animal].getDictionnary()

        for idAnimalB in range( 1 , 5 ):
            if ( animal == idAnimalB ):
                continue
             
            approachDico[animal, idAnimalB] = EventTimeLineCached( connection, file, "Social approach", animal, idAnimalB, minFrame=tmin, maxFrame=tmax ) #fait une matrice de toutes les aproches à deux possibles

    for animal in range( 1 , 5 ):
        
        for idAnimalB in range( 1 , 5 ):
            if( animal == idAnimalB ):
                continue
            
            eventName = "Approach rear"        
            # print ( eventName )
            
            appRearTimeLine = EventTimeLine( None, eventName , animal , idAnimalB , None , None , loadEvent=False )
            
            
            for eventApp in approachDico[animal, idAnimalB].eventList:
                
                ''' new code: '''
                
                for t in range( eventApp.endFrame - TIME_WINDOW_BEFORE_EVENT, eventApp.endFrame + TIME_WINDOW_BEFORE_EVENT + 1 ):
                    
                    if ( t in dicoEventRearAnimal[idAnimalB] ):
                        appRearTimeLine.eventList.append(eventApp)
                        break
                        
                '''
                previous(slow) code
                
                for eventRear in rearDico[idAnimalB].eventList:
                    if (eventRear.overlapInT(eventApp.endFrame-TIME_WINDOW_BEFORE_EVENT, eventApp.endFrame+TIME_WINDOW_BEFORE_EVENT) == True):
                        appRearTimeLine.eventList.append(eventApp)
                '''
                
            appRearTimeLine.endRebuildEventTimeLine(connection)
            
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Approach Rear" , tmin=tmin, tmax=tmax )

       
    # print( "Rebuild event finished." )
        
    


'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.EventTimeLineCache import EventTimeLineCached

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "Get away" )


def reBuildEvent( connection, file, tmin=None, tmax=None, pool = None ):
    '''    
    Get away: at least 1 animal should move, mouse A speed > mouse B speed & mouse A getting away from B.

    ''' 
    
    # print("Escape")
    
    ''' use the pool provided or create it'''
    if ( pool == None ):
        pool = AnimalPool( )
        pool.loadAnimals( connection )
        pool.loadDetection( start = tmin, end = tmax )
    
    nbAnimal = pool.getNbAnimals()

    
            
    for animal in range( 1 , nbAnimal+1 ):
        
        for idAnimalB in range( 1 , nbAnimal+1 ):
            if( animal == idAnimalB ):
                continue
                        
            eventName = "Get away"        
            # print ( eventName )
            
            getAwayTimeLine = EventTimeLine( None, eventName , animal , idAnimalB , None , None , loadEvent=False )

            result={}
            
            for t in range ( tmin, tmax+1 ):                
            
                animalA = pool.animalDictionnary[animal]
                animalB = pool.animalDictionnary[idAnimalB]
                
                if not ( t-1 in animalA.detectionDictionnary ):
                    continue                
                if not ( t+1 in animalA.detectionDictionnary ):
                    continue

                if not ( t-1 in animalB.detectionDictionnary ):
                    continue                
                if not ( t+1 in animalB.detectionDictionnary ):
                    continue
                                            
                speedA = animalA.getSpeed( t )
                speedB = animalB.getSpeed( t )
                
                if ( speedB > speedA ):
                    continue
                
                if ( speedA > SPEED_THRESHOLD_LOW or speedB > SPEED_THRESHOLD_LOW ):
                    
                    dAStart = animalA.detectionDictionnary[t-1]
                    dAEnd = animalA.detectionDictionnary[t+1]
                    dBStart = animalB.detectionDictionnary[t-1]
                    dBEnd = animalB.detectionDictionnary[t+1]
                    
                    distStart = dAStart.getDistanceTo( dBStart )
                    distEnd = dAEnd.getDistanceTo( dBEnd )
                    
                    if distStart != None and distEnd != None:
                        if( distStart < distEnd ):
                            result[t] = True
            
            
            getAwayTimeLine.reBuildWithDictionnary( result )
            
            getAwayTimeLine.endRebuildEventTimeLine(connection)
        
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Get Away" , tmin=tmin, tmax=tmax )

             
    # print( "Rebuild event finished." )
        
    


'''
Created on 12 sept. 2017

@author: Fab
'''

import math
from lmtanalysis.Measure import *

class Detection():

    __slots__ = ('massX', 'massY','massZ','massPoint','frontX','frontY','frontZ','frontPoint','backX','backY','backZ','backPoint','rearing','lookUp','lookDown','mask','frame' )

    def __init__(self, massX, massY, massZ=None, frontX=None, frontY=None, frontZ=None, backX=None, backY=None, backZ=None, rearing=None, lookUp=None, lookDown=None , lightLoad = False , frame = None ):
        
        self.massX = massX
        self.massY = massY
        self.frame = frame
        
        if lightLoad:
            return
        
        self.massZ = massZ
        self.massPoint = Point( massX , massY )
            
        self.frontX = frontX
        self.frontY = frontY
        self.frontZ = frontZ
        self.frontPoint = Point( frontX , frontY )

        self.backX = backX
        self.backY = backY
        self.backZ = backZ
        self.backPoint = Point( backX , backY )

        self.rearing = rearing
        self.lookUp = lookUp
        self.lookDown = lookDown
        
    
    def setMask( self, mask ):
        self.mask = mask
        
    def isHeadAndTailDetected(self):
        
        if ( self.frontX == -1 or self.frontY == -1 or self.backX == -1 or self.backY == -1 ):
            return False
        
        return True
    
    def getBodySize(self):
        return math.hypot(self.frontX-self.backX, self.frontY-self.backY)
    
    
    def getBodySlope(self):
        '''
        calculate the instantaneous slope of the animal between nose and tail
        '''
        
        if (self.frontZ==0 or self.backZ==0):
            return None
        else:  
            bodySlope = (self.frontZ - self.backZ)
        
        return bodySlope
    
    def getMassCenterPoint(self):
        return Point( self.massX , self.massY )

    def getFrontPoint(self):
        return Point( self.frontX , self.frontY )
    
    def getBackPoint(self):
        return Point( self.backX , self.backY )
    
    def getDirection(self):
        '''
        determines the direction of the animal using the head and the mass center
        '''
        #angleDir = math.atan2(self.frontY-self.massY, self.frontX-self.massX)
        angleDir = math.atan2(self.frontY-self.backY, self.frontX-self.backX)
        return angleDir
            
    
    def getDistanceTo (self, detectionB):
        '''
        determine the distance between the focal animal and animalB at one specified time point t
        check before that both animals are detected at this time point
        '''
        distanceTo = None

        if ( detectionB == None ):
            return None
        
        if (detectionB.massX == None):
            return None
        
        if (math.hypot( self.massX - detectionB.massX, self.massY - detectionB.massY ) > MAX_DISTANCE_THRESHOLD): #if the distance calculated between the two individuals is too large, discard
            # print("WARNING: Detection.getDistanceTo : Distance Max reached. returning None")
            return None
        
        else:
            distanceTo = math.hypot( self.massX - detectionB.massX, self.massY - detectionB.massY )
            return distanceTo
    
    
    def getDistanceToPoint (self, xPoint, yPoint):
        '''
        determine the distance between the focal animal and a specific point in the arena
        '''

        distanceToPoint = math.hypot( self.massX - xPoint, self.massY - yPoint )
        return distanceToPoint
    
    
    def isInZone (self, xa=149, xb=363, ya=98, yb=318):
        '''
        check whether a detection of animal A is located in the specified zone of the cage
        Default zone is the center xa=149, xb=363, ya=318, yb=98
        '''
        x1 = min( xa, xb )
        x2 = max( xa, xb )
        y1 = min( ya, yb )
        y2 = max( ya, yb )
        
        if ( self.massX > x1 and self.massX < x2 and self.massY > y1 and self.massY < y2 ):
            return True
        
        return False
    
    
    def isRearing(self):
        '''
        determine whether the animal is rearing at this detection
        '''
        if (self.getBodySlope() == None):
            return False
                
        if (self.getBodySlope() > -BODY_SLOPE_THRESHOLD and self.getBodySlope() < BODY_SLOPE_THRESHOLD):
            return False
        
        else:
            return True
        
    def isRearingZ(self):
        '''
        determine whether the animal is rearing at this detection, using the old criteria from the first version
        '''
        if (self.frontZ == None):
            return False
                
        if (self.frontZ < FRONT_REARING_THRESHOLD):
            return False
        
        else:
            return True


'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.EventTimeLineCache import EventTimeLineCached

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "Other contact" )

def reBuildEvent( connection, file, tmin=None, tmax=None , pool = None ): 

    if ( pool == None ):
        pool = AnimalPool( )
        pool.loadAnimals( connection )
        #pool.loadDetection( start = tmin, end = tmax , lightLoad=True )

    ''' load the contact timeline for each pair of animals '''
    contactTimeLine = {}
    otherContactTimeLine = {}
    for animal in range( 1 , pool.getNbAnimals()+1 ):
        for idAnimalB in range( 1 , pool.getNbAnimals()+1 ):
            if ( animal == idAnimalB ):
                continue
            contactTimeLine[animal, idAnimalB] = EventTimeLineCached( connection, file, "Contact", animal, idAnimalB, minFrame=tmin, maxFrame=tmax )

            # create an empty timeline for other contacts
            otherContactTimeLine[animal, idAnimalB] = EventTimeLine(None, 'Other contact', animal, idAnimalB, loadEvent=False)

    ''' load the timelines of the different specific contacts '''
    timeLine = {}
    for event in ["Oral-oral Contact", "Oral-genital Contact", "Passive oral-genital Contact", "Side by side Contact", "Side by side Contact, opposite way"]:
        timeLine[event] = {}
        for animal in range(1, pool.getNbAnimals() + 1):
            for idAnimalB in range(1, pool.getNbAnimals() + 1):
                if (animal == idAnimalB):
                    continue
                timeLine[event][animal, idAnimalB] = EventTimeLineCached(connection, file, event, animal, idAnimalB, minFrame=tmin, maxFrame=tmax)
                '''# clean the specific contact timelines:
                timeLine[event][animal, idAnimalB].mergeCloseEvents(numberOfFrameBetweenEvent=1)
                timeLine[event][animal, idAnimalB].removeEventsBelowLength(maxLen=3)'''

    contactDico = {}
    for animal in range(1, pool.getNbAnimals() + 1):
        for idAnimalB in range(1, pool.getNbAnimals() + 1):
            if (animal == idAnimalB):
                continue
            contactDico[animal, idAnimalB] = contactTimeLine[animal, idAnimalB].getDictionary(minFrame=tmin, maxFrame=tmax)

            eventDico = {}
            for event in ["Oral-oral Contact", "Oral-genital Contact", "Passive oral-genital Contact", "Side by side Contact", "Side by side Contact, opposite way"]:
                eventDico[event] = timeLine[event][animal, idAnimalB].getDictionary(minFrame=tmin, maxFrame=tmax)
                '''remove the frames where the animals are in specific contacts'''
                for t in eventDico[event].keys():
                    contactDico[animal, idAnimalB].pop(t, None)

            otherContactTimeLine[animal, idAnimalB].reBuildWithDictionnary(contactDico[animal, idAnimalB])
            otherContactTimeLine[animal, idAnimalB].endRebuildEventTimeLine(connection)


    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Other contact" , tmin=tmin, tmax=tmax )
                       
    # print( "Rebuild event finished." )
        
    


import sys
sys.path.insert(1, "../")

import pandas as pd
import warnings
warnings.simplefilter(action='ignore')
import os
import glob
import csv
import shutil
import tkinter as tk
from tkinter.dialog import Dialog
from tkinter import filedialog
from tkinter.filedialog import askdirectory, askopenfilename

def window():
    root = tk.Tk()
    root.withdraw()
    root.update()

    # Utilisez Dialog() pour afficher une boîte de dialogue de sélection
    d = Dialog(
        title="Select folder or cancel",
        text="Select folder or cancel",
        bitmap='question',
        strings=('Folder', 'Cancel'),
        default=0
    )

    # Si l'utilisateur choisit 'Cancel', terminez la fonction
    if d.num == 1:
        root.destroy()
        return None

    folder = askdirectory(title="Choose a folder where your .csv files to merge are located")
    root.destroy()
    return folder

def Merge():
    """
    This function merges all CSV files in a selected folder into a single CSV file.

    Instructions:
    1. A dialog box opens to select a folder containing CSV files.
    2. Specify the name of the new folder to create where the files will be moved.
    3. CSV files from the selected folder will be moved to the new folder.
    4. All CSV files in the new folder will be merged into a single CSV file.
    5. The new merged file will be saved in the new folder with a specified name.

    Note:
    - Ensure that the os, shutil, glob, and pandas modules are imported.
    - If you have a column "Unnamed: 0" in your CSV files, use the line `df_append.pop("Unnamed: 0")`
      to remove it before merging the files.
    - Make sure your code calls the window() function to display the folder selection dialog.

    Example Usage:
    Merge()
    """

    # Open a dialog box to select a folder
    path = window()

    # Specify the name of the new folder to create
    input_new_folder = input("Enter the name of the new folder to create: ")
    new_folder = input_new_folder

    # Specify the path of the folder where the .csv files are located
    root = path

    # Specify the path of the folder where the new folder will be created
    final_file = path

    # Check if the folder already exists
    if os.path.exists(os.path.join(final_file, new_folder)):
        # If yes, remove it
        shutil.rmtree(os.path.join(final_file, new_folder))

    # Create the new folder in the destination folder
    new_path = os.path.join(final_file, new_folder)
    os.makedirs(new_path)

    # Iterate through all files in the initial folder
    for file in os.listdir(root):

        # Check if the file is a .csv file
        if file.endswith(".csv"):

            # Specify the full path of the file
            path2 = os.path.join(root, file)

            # Move the file to the new folder
            shutil.move(path2, new_path)

    # read the path
    file_path = path+"/"+new_folder+"/"
    os.chdir(path+"/"+new_folder+"/")
    # list all the files from the directory
    file_list = os.listdir(file_path)

    #list all csv files only
    csv_files = glob.glob('*.{}'.format('csv'))
    print(csv_files)

    df_append = pd.DataFrame()  # append all files together

    for file in csv_files:
        df_temp = pd.read_csv(file)
        df_append = df_append.append(df_temp, ignore_index=True)

    print("##################################################################################")
    print("##################################################################################")
    print("######################### Folder created & files moved ###########################")

    df_append.pop("Unnamed: 0") #Add this line if there is a column named "Unnamed: 0"
    name_new_file = input("What's the name of your merged file ?")
    df_append.to_csv(path+"/"+new_folder+"/"+name_new_file+".csv")

    print("##################################################################################")
    print("##################################################################################")
    print("####################### Merged file created with success #########################")



'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "SAP" )

def reBuildEvent( connection, file, tmin=None, tmax=None, pool = None , showGraph = False ): 
    
    ''' use the pool provided or create it'''
    if ( pool == None ):
        pool = AnimalPool( )
        pool.loadAnimals( connection )
        pool.loadDetection( start = tmin, end = tmax )
    
    
    ''' 
    Animal A is stopped (built-in event):
    Move social: animal A is stopped and in contact with any other animal.
    Move isolated: animal A is stopped and not in contact with any other animal.
    ''' 
        
    
    for animal in pool.animalDictionnary:
        
        animal = pool.animalDictionnary[animal]
                
        SAPTimeLine = EventTimeLine( connection, "SAP", animal.baseId, minFrame=tmin, maxFrame=tmax, loadEvent=False )

        #f = animal.getCountFramesSpecZone( start , start+oneMinute*30 , xa=143, ya=190, xb=270, yb=317 )
        result = animal.getSapDictionnary( tmin , tmax )
            
        SAPTimeLine.reBuildWithDictionnary( result )
        SAPTimeLine.endRebuildEventTimeLine(connection)
        #animal.clearDetection()
    
        
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event SAP" , tmin=tmin, tmax=tmax )

               
    # print( "Rebuild event finished." )
        
    


'''
Created on 6 sept. 2017

@author: Fab

In some record, we do have an extra animal with None for all parameters.
This script should be used to detect those animal in databases.

'''
import sqlite3
from time import *

from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *
from lmtanalysis.Chronometer import Chronometer


def check( connection, tmin=None, tmax=None ): 
    
    pool = AnimalPool( )
    pool.loadAnimals( connection )

    '''
    get the number of expected animals
    '''
    
    nbAnimals = pool.getNbAnimals()
    # print("nb animals: " , nbAnimals )
    # for animal in pool.getAnimalList():
    #     if ( animal.name == None ):
    #         print( "!!!! None animal detected with lmtanalysis id: " , animal.baseId )
    
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Correct wrong animal" , tmin=tmin, tmax=tmax )

       
    # print( "Rebuild event finished." )
        
    


'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
from time import *
from lmtanalysis.Chronometer import Chronometer
from lmtanalysis.Animal import *
from lmtanalysis.Detection import *
from lmtanalysis.Measure import *
import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Event import *
from lmtanalysis.Measure import *

def flush( connection ):
    ''' flush event in database '''
    deleteEventTimeLineInBase(connection, "Oral-genital Contact" )

def distHeadBack( detA, detB ):
    
    hx = detA.frontX
    hy = detA.frontY
    bx = detB.backX
    by = detB.backY
    
    if ( hx == -1 or hy == -1 or bx == -1 or by == -1 ):
        return 100000
    
    return math.hypot( hx-bx, hy-by )
    
def reBuildEvent( connection, file, tmin=None, tmax=None, pool = None ): 
    
    ''' use the pool provided or create it'''
    if ( pool == None ):
        pool = AnimalPool( )
        pool.loadAnimals( connection )
        pool.loadDetection( start = tmin, end = tmax )
    
    for animal in range( 1 , pool.getNbAnimals()+1 ):
        
        for idAnimalB in range( 1 , pool.getNbAnimals()+1 ):
            if( animal == idAnimalB ):
                continue
            
            ''' MAX_DISTANCE_HEAD_HEAD_GENITAL_THRESHOLD '''
            
            eventName = "Oral-genital Contact"        
            # print ( eventName )
            
            result ={}
            animalA = pool.animalDictionnary.get( animal )
            animalB = pool.animalDictionnary.get( idAnimalB )            
            
            OralGenitalTimeLine = EventTimeLine( None, eventName , animal , idAnimalB , loadEvent=False )

            for t in animalA.detectionDictionnary.keys() :
                
                if ( t in animalB.detectionDictionnary ):
                    detA = animalA.detectionDictionnary[t]
                    detB = animalB.detectionDictionnary[t]
                    
                    if distHeadBack( detA, detB ) < MAX_DISTANCE_HEAD_HEAD_GENITAL_THRESHOLD:
                        result[t] = True
            
            OralGenitalTimeLine.reBuildWithDictionnary( result )
            OralGenitalTimeLine.endRebuildEventTimeLine(connection)
    
        
    # log process
    from lmtanalysis.TaskLogger import TaskLogger
    t = TaskLogger( connection )
    t.addLog( "Build Event Oral Genital Contact" , tmin=tmin, tmax=tmax )
        
                   
    # print( "Rebuild event finished." )
        
    


'''
Created on 6 sept. 2017

@author: Fab
'''
import sqlite3
import unittest
from time import *
from lmtanalysis.Chronometer import Chronometer

import matplotlib
#matplotlib fix for mac
#matplotlib.use('TkAgg')

import matplotlib.pyplot as plt
import numpy as np
from lmtanalysis.Measure import *
import sys
import json

class Event:
    '''
    an event represent the interval of frame where the event is
    '''
    def __init__(self , startFrame, endFrame, baseId = None, metadata=None, description=None ):

        self.startFrame = startFrame
        self.endFrame = endFrame
        self.description = description

        if metadata==None:
            self.metadata = {}
        else:
            self.metadata = json.loads( metadata )

        if baseId!= None:
            self.baseId = baseId

    def updateMetaData(self, connection , commit= False ):
        
        #print("Update metadata")
        toStore = json.dumps( self.metadata )
        #print( toStore )
        #print( self.baseId )

        #query = "UPDATE EVENT SET METADATA=%s WHERE ID=%s".format( toStore, self.baseId )
        query = "UPDATE EVENT SET METADATA=? WHERE ID=?"
        #print ( query )
        c = connection.cursor()
        c.execute( query , (toStore , self.baseId ) )
        if commit:
            connection.commit()



    def duration(self):
        return self.endFrame-self.startFrame+1

    def contain(self , frameNumber ):
        return ( frameNumber >= self.startFrame and frameNumber <= self.endFrame )

    def overlapInT(self , start , end ):
        '''
        XXXX
          XXXX

         YYYY
        YYYYYY
        '''
        if ( start <= self.endFrame and end >= self.startFrame ):
            return True
        #if ( end <= self.endFrame and end >= self.startFrame ):
        #    return True
        #if ( start <= self.endFrame and start >= self.startFrame ):
        #    return True
        ## case where the start and end are over boundaries of event
        #if ( start <= self.startFrame and end >= self.endFrame ):
        #    return True
        #return False

    def overlapEvent(self , eventCandidate ):
        result = self.overlapInT( eventCandidate.startFrame , eventCandidate.endFrame )
        #print ( "Overlap debug: ( {} , {} ) - ( {} , {} ) : {}".format( self.startFrame, self.endFrame , eventCandidate.startFrame, eventCandidate.endFrame, result )  );
        return result

    def shift(self, nbFrame):
        '''
        Shifts event of nbFrame
        '''
        self.startFrame += nbFrame
        self.endFrame += nbFrame

    def numberOfFrameToEvent(self , eventCandidate ):
        '''
        provides the distance (in frame) to another event
        if the events do overlap, return 0
        '''

        if ( self.overlapEvent( eventCandidate ) ):
            return 0;

        return min( abs( self.startFrame - eventCandidate.endFrame ), abs( self.endFrame - eventCandidate.startFrame ) )

    def __str__(self):
        return "Event\tstart:\t"+ str( self.startFrame) + "\tend:\t" + str( self.endFrame)

class EventTimeLine:
    '''
    classdocs
    '''
    def __init__(self, conn, eventName, idA=None , idB=None ,idC=None , idD=None , loadEvent=True, minFrame = None, maxFrame=None, inverseEvent = False , loadEventIndependently = False ):
        '''
        load events
            where t>=minFrame and t<=maxFrame if applicable
            inverseEvent: inverse all timeline (used to make stop becomes move for instance)
        '''

        #check id at 0 to transform them as None ( id will not be considered in query ) (added for USV support).

        if ( idA == 0 ):
            idA = None
        if ( idB == 0 ):
            idB = None
        if ( idC == 0 ):
            idC = None
        if ( idD == 0 ):
            idD = None

        #store parameters
        self.idA = idA
        self.idB = idB
        self.idC = idC
        self.idD = idD
        self.eventName = str( eventName )
        self.eventNameWithId = "{} idA:{} idB:{} idC:{} idD:{}".format( eventName , idA , idB, idC, idD )
        # build events
        self.eventList = []

        if ( loadEvent == False ):
            # print( "Event " + str( eventName ) + " created. eventNameWithId = " +  str( self.eventNameWithId ) + " loadEvent: False" )
            return;

        chrono = Chronometer( "Load event " + str ( self.eventName ) )
        c = conn.cursor()

        query = "SELECT * FROM EVENT WHERE NAME='{0}'".format( self.eventName );
        if ( idA != None ):
            query += " AND IDANIMALA={0}".format( idA )

        if ( idB != None ):
            query += " AND IDANIMALB={0}".format( idB )

        if ( idC != None ):
            query += " AND IDANIMALC={0}".format( idC )

        if ( idD != None ):
            query += " AND IDANIMALD={0}".format( idD )

        '''
        if ( minFrame != None ):
            query += " AND STARTFRAME>={0}".format( minFrame )

        if ( maxFrame != None ):
            query += " AND ENDFRAME<={0}".format( maxFrame )
        '''

        if ( minFrame != None ):
            query += " AND ENDFRAME>={0}".format( minFrame )

        if ( maxFrame != None ):
            query += " AND STARTFRAME<={0}".format( maxFrame )

        query += " ORDER BY STARTFRAME"

        ''' print( query ) '''
        c.execute( query )
        all_rows = c.fetchall()

        if loadEventIndependently == True: # load events without using the rebuild dictionnary.

            if inverseEvent == True:
                print ( "Warning: inverseEvent option not compatible in loadEventWithoutOverlapCheck")

            for row in all_rows:

                start = row[3]
                end = row[4]

                if ( minFrame != None ):
                    if ( start < minFrame or end < minFrame ):
                        continue

                if ( maxFrame != None ):
                    if ( start > maxFrame or end > maxFrame ):
                        continue

                metadata = None
                if len(row) >=10 :
                    #print (row )
                    metadata = row[9]
                    #print("Load meta data: " , metadata )

                self.eventList.append( Event( start, end , metadata = metadata, baseId = row[0], description=row[2] ) )


        else:

            eventBool = {}

            for row in all_rows:
                start = row[3]
                end = row[4]
                for t in range( start, end+1 ):

                    if ( minFrame != None ):
                        if ( t < minFrame ):
                            continue

                    if ( maxFrame != None ):
                        if ( t > maxFrame ):
                            continue

                    eventBool[t] = True

            if ( inverseEvent == True ):

                if ( minFrame == None ):
                    # print("To inverse event, need a minFrame")
                    return
                if ( maxFrame == None ):
                    # print("To inverse event, need a maxFrame")
                    return

                for t in range( minFrame , maxFrame +1 ):
                    if ( t in eventBool ):
                        eventBool.pop( t )
                    else:
                        eventBool[t] = True

            self.reBuildWithDictionnary(eventBool)

        #keyList = sorted(eventBool.keys())

        #start = -1
        #for key in keyList:

        #    if ( start == -1 ):
        #        start = key

        #    if ( eventBool.get( key+1 ) == None ):
        #        self.eventList.append( Event( start, key ) )
        #        start = -1

        # print ( eventName , " Id(",idA ,",", idB, ",", idC, "," , idD , ") Min/maxFrame: (",minFrame,"/",maxFrame ,") Loaded (" , len( self.eventList ) , " records loaded in ", chrono.getTimeInS() , "S )")

    def __str__(self):
        return self.eventName + " Id(" + str(self.idA) + ","+  str(self.idB)+ ","+ str(self.idC)+ "," + str(self.idD) + ")"

    def saveTimeLine(self , conn , saveDescriptionPerEvent=False ):
        c = conn.cursor()
        for event in self.eventList:
            query = "INSERT INTO EVENT (NAME, DESCRIPTION, STARTFRAME, ENDFRAME, IDANIMALA, IDANIMALB, IDANIMALC, IDANIMALD, METADATA ) VALUES (?,?,?,?,?,?,?,?,?);"

            jsonToStore = json.dumps( event.metadata )
            description = "" # self.eventNameWithId
            if saveDescriptionPerEvent:
                description = event.description
            c.execute( query , ( self.eventName, description, event.startFrame, event.endFrame , self.idA, self.idB, self.idC, self.idD, jsonToStore ) )
        conn.commit()


    def addEvent(self , eventToAdd , noCheck =False ):
        self.eventList.append ( eventToAdd )
        if not noCheck :
            self.checkEventHole( eventToAdd.startFrame-1 )
            self.checkEventHole( eventToAdd.endFrame )

            # check merge with existing events
            for event in self.eventList[:]:
                if ( event == eventToAdd ):
                    continue
                if ( event.overlapEvent( eventToAdd ) ):
                    self.mergeEvent( event, eventToAdd )

    def addPunctualEvent(self , frameNumber ):
        '''
        Adds an event at frame frameNumber. If an event exists beside, extends it, else creates it.
        '''
        #check if event exists
        if ( self.getEventAt( frameNumber ) != None ):
            return;

        for event in self.eventList[:]:
            if ( event.endFrame == frameNumber-1 ):
                event.endFrame = frameNumber
                self.checkEventHole( frameNumber )
                return

        for event in self.eventList[:]:
            if ( event.startFrame == frameNumber + 1 ):
                event.startFrame = frameNumber
                self.checkEventHole( frameNumber -1 )
                return

        #create the event

        event = Event( frameNumber , frameNumber )
        self.eventList.append( event )

    def getEventAt(self , frameNumber ):
        for event in self.eventList:
            if ( event.contain( frameNumber ) ):
                return event
        return None

    def checkIfEventListIsOrdered(self ):
        '''
        Checks if the eventlist is fully ordered
        '''
        cursor = -1
        previousEvent = None
        for event in self.eventList:
            if ( event.startFrame > cursor ):
                cursor = event.endFrame
            else:
                # print( "Check ordered list: problem in list:")
                # print( "previous event : ", previousEvent )
                # print( "next (non ordered) event event : ", event )
                return False
            previousEvent = event

        return True

    '''
    returns the closest event from framenumber, and the distance (in frame) to it.
    assumes the eventList has been build by rebuildallevents, which makes them ordered.
    '''
    def  getClosestEventFromFrame(self , frameNumber , optimizeAssumingOrderedList = False , constraint = None ):

        if ( self.hasEvent( frameNumber )):
            # we do have an event for this frame so the distance is 0
            return None, 0

        listToSearchIn = self.eventList

        '''
        todo: fix this algorithm. removed from process (no impact on result):
        # pre-fetch events.
        if ( optimizeAssumingOrderedList == True ):
            listToSearchIn=[]
            for event in self.eventList:                
                # we just overtook the best match.
                # we put the current event and the previous one (if it exists) in the list
                currentIndex = self.eventList.index( event )
                listToSearchIn.append( event )
                if ( currentIndex > 0 ):
                    listToSearchIn.append( self.eventList[currentIndex-1] )
                break
        '''

        if constraint=="after frame":
            # creates a new list where only events in the future are kept
            updatedList = []
            for event in listToSearchIn:
                if ( event.startFrame > frameNumber ):
                    updatedList.append( event )
            listToSearchIn = updatedList

        if constraint=="before frame":
            # creates a new list where only events in the past are kept
            updatedList = []
            for event in listToSearchIn:
                if ( event.endFrame < frameNumber ):
                    updatedList.append( event )
            listToSearchIn = updatedList

        closestEvent = None
        bestDistance = sys.maxsize
        
        for event in listToSearchIn:

            distanceToStart = abs( frameNumber - event.startFrame )
            distanceToEnd = abs( frameNumber - event.endFrame )
            best = min( distanceToStart, distanceToEnd )
            #print( bestDistance, best )
            if ( best < bestDistance ):
                closestEvent = event
                bestDistance = best                

        return closestEvent , bestDistance

    def hasEvent(self , frameNumber):
        if ( self.getEventAt(frameNumber) != None ):
            return True
        return False

    def getNumberOfEvent(self , minFrame=None, maxFrame=None ):
        nbEvent = 0

        for event in self.eventList:
            takeEvent = True

            if ( minFrame != None ):
                if ( event.endFrame < minFrame ):
                    takeEvent = False

            if ( maxFrame != None ):
                if ( event.startFrame > maxFrame ):
                    takeEvent = False

            if (takeEvent):
                nbEvent+=1

        return nbEvent


    def getTotalDurationEvent( self, tmin, tmax ):
        '''
        return total duration between min and max t value.
        '''
        duration = 0
        dicEvent = self.getDictionnary()
        for t in range( tmin, tmax + 1):
            if t in dicEvent:
                duration+=1

        return duration

    def getDurationEventInTimeBin(self, tmin=0, tmax=None, binSize=1*oneMinute):
        '''
        compute the proportion of frames within an event within a given time bin, to calculate the density
        '''
        if ( tmax == None ):
            tmax = self.getMaxT()

        dicEvent = self.getDictionnary()

        durationEventInBinProportionList = []

        frame = tmin

        while (frame < tmax):

            durationEventInBin = 0

            for t in range(frame, frame + binSize):
                if (t in dicEvent.keys()):
                    durationEventInBin = durationEventInBin + 1

            durationEventInBinProportion = durationEventInBin/binSize

            durationEventInBinProportionList.append(durationEventInBinProportion)

            frame = frame + binSize

        return durationEventInBinProportionList

    '''
    deprecated
    '''
    def getDictionnary(self , minFrame=None, maxFrame=None ):
        frameDico = {}
        for event in self.eventList:
            for t in range( event.startFrame, event.endFrame +1):
                frameDico[t] = True;

        if ( minFrame !=None ):
            for key in dict ( frameDico ).keys() :
                if ( key < minFrame ):
                    frameDico.pop( key )

        if ( maxFrame !=None ):
            for key in dict ( frameDico ).keys():
                if ( key > maxFrame ):
                    frameDico.pop( key )


        return frameDico

    def getDictionary(self , minFrame=None, maxFrame=None ):
        frameDico = {}
        for event in self.eventList:
            for t in range( event.startFrame, event.endFrame +1):
                frameDico[t] = True;

        if ( minFrame !=None ):
            for key in dict ( frameDico ).keys() :
                if ( key < minFrame ):
                    frameDico.pop( key )

        if ( maxFrame !=None ):
            for key in dict ( frameDico ).keys():
                if ( key > maxFrame ):
                    frameDico.pop( key )


        return frameDico

    def clearEvents(self):
        self.eventList.clear()

    def reBuildWithDictionnary(self, eventBool ):

        self.eventList.clear()

        keyList = sorted(eventBool.keys())

        start = -1
        for key in keyList:

            if ( start == -1 ):
                start = key

            if ( eventBool.get( key+1 ) == None ):
                self.eventList.append( Event( start, key ) )
                start = -1


    def checkEventHole( self, frameNumber ):
        '''
        Checks if an event end at a givenFrame, and if another one starts just after. Merge event if found.
        XXX
          T
           XXX
           ==> merge
        '''
        eventA = self.getEventAt( frameNumber )
        eventB = self.getEventAt( frameNumber +1)
        if ( eventA == None or eventB == None ):
            return
        if ( eventA != eventB ):
            self.mergeEvent( eventA, eventB )

    def mergeCloseEvents( self, numberOfFrameBetweenEvent ):
        '''
        Merges events if they are distant of less or equal than numberOfFrame
        '''
        minT = self.getMinT()
        maxT = self.getMaxT( )

        if ( self.getNbEvent() == 0 ):
            # print("No event in timeLine")
            return

        eventDictionnary = self.getDictionnary( minT, maxT )

        nbFrameOutOfEvent = 0
        lastEventEndFrame = minT
        for t in range( minT , maxT+1 ):
            #print( "t: " + str(t) )
            if t in eventDictionnary:

                if nbFrameOutOfEvent > 0:
                    if nbFrameOutOfEvent <= numberOfFrameBetweenEvent:
                        # fill
                        for tt in range ( lastEventEndFrame , t ):
                            #print( "filling " + str( tt ))
                            eventDictionnary[tt] = True

                nbFrameOutOfEvent = 0
                lastEventEndFrame = t
            else:
                nbFrameOutOfEvent += 1

        self.reBuildWithDictionnary( eventDictionnary )

    def dilateEvents(self, numberOfFrame):
        '''
        Dilate events in time.
        '''
        '''
        minT = self.getMinT()
        maxT = self.getMaxT( )
        '''

        if ( self.getNbEvent() == 0 ):
            # print("No event in timeLine")
            return

        #eventDictionnary = self.getDictionnary( minT, maxT )
        eventDictionnaryDilated = {}

        for event in self.eventList:            
            for tt in range ( event.startFrame - numberOfFrame, event.endFrame +numberOfFrame+1 ):
                eventDictionnaryDilated[tt]=True
            

        '''
        #faster algo:
        
        # copy original to dilated
        eventDictionnaryDilated = eventDictionnary.copy()
        for d in range( numberOfFrame ):
            print("Dilate " , d)
            for t in eventDictionnaryDilated.copy().keys():    
                
                if t-1 not in eventDictionnaryDilated:
                    eventDictionnaryDilated[t-1] = True
                    
                if t+1 not in eventDictionnaryDilated:
                    eventDictionnaryDilated[t+1] = True
                
        '''
        
        
        '''
        #slow:
        for t in eventDictionnary.keys():
        #for t in range( minT , maxT+1 ):
            for tt in range ( t - numberOfFrame, t+numberOfFrame+1 ):
                eventDictionnaryDilated[tt]=True
        '''
        
        self.reBuildWithDictionnary( eventDictionnaryDilated )



    def mergeEvent( self , eventA, eventB ):
        '''
        if events are next to another, merge it. (test A before B and B before A )
        '''
        if( eventA.endFrame == eventB.startFrame-1 ):
            mergedEvent = Event( eventA.startFrame , eventB.endFrame )
            self.eventList.remove( eventA )
            self.eventList.remove( eventB )
            self.eventList.append( mergedEvent )
            return

        if( eventB.endFrame == eventA.startFrame-1 ):
            mergedEvent = Event( eventB.startFrame , eventA.endFrame )
            self.eventList.remove( eventA )
            self.eventList.remove( eventB )
            self.eventList.append( mergedEvent )
            return

        if ( eventA.overlapEvent( eventB ) ):
            mini = min( eventA.startFrame , eventB.startFrame )
            maxi = max( eventA.endFrame , eventB.endFrame )
            #print ( "mini: " , mini )
            #print ( "maxi: " , maxi )
            mergedEvent = Event( mini , maxi )
            try:
                self.eventList.remove( eventA )
            except:
                pass
            try:
                self.eventList.remove( eventB )
            except:
                pass

            self.eventList.append( mergedEvent )
            return

    def getEventList(self):
        return self.eventList

    def removeEventsOverT(self , maxT ):
        for event in self.eventList[:]:
            if ( event.startFrame > maxT ):
                self.eventList.remove( event )

    def removeEventsBelowT(self , maxT ):
        for event in self.eventList[:]:
            if ( event.endFrame < maxT ):
                self.eventList.remove( event )

    def overlap(self , candidateEvent, dictionary = None ):
        '''
        if the user provides the dictionary, this call is much faster
        '''
        if dictionary == None:
            dictionary = self.getDictionnary()
        for t in range ( candidateEvent.startFrame, candidateEvent.endFrame+1):
            if t in dictionary:
                return True
        return False

    def keepOnlyEventCommonWithTimeLine(self , timeLineAnd ):
        '''
        perform a AND logic with timeLineAnd
        '''
        dico = self.getDictionnary()
        andDico = timeLineAnd.getDictionnary()
        andResult = {}

        for k in dico:
            if k in andDico:
                andResult[k] = True

        self.reBuildWithDictionnary( andResult )


    def removeEventOfTimeLine(self , timeLineToRemove ):
        '''
        remove events that match timeLineToRemove
        '''
        dico = self.getDictionnary()
        removeDico = timeLineToRemove.getDictionnary()

        for k in removeDico:
            dico.pop( k , None )

        self.reBuildWithDictionnary( dico )


    def removeEventsBelowLength(self , maxLen ):
        for event in self.eventList[:]:
            if ( event.duration() < maxLen ):
                self.eventList.remove( event )

    def printEventList(self):
        # print( "Event list of {} / {} / nbEvent: {}".format( self.eventName , self.eventNameWithId, len( self.eventList ) ) )
        i = 0
        for event in self.eventList:
            # print ( "[{}] Event ({},{})".format( i, event.startFrame, event.endFrame ) )
            i+=1

    def getMaxT(self):
        maxT = None
        for event in self.eventList:
            if maxT == None:
                maxT = event.endFrame
            maxT = max( maxT , event.endFrame )
        return maxT

    def getMinT(self):
        minT = None
        for event in self.eventList:
            if minT == None:
                minT = event.startFrame
            minT = min( minT , event.startFrame )
        return minT

    def getNbEvent(self):
        nb=0

        for event in self.eventList:
            nb = nb+1

        return nb

    def getEventLengthList(self):
        eventLengthList = []
        for event in self.eventList:
            eventLengthList.append(event.duration())

        return eventLengthList

    def getMeanEventLength(self):
        nb=0
        sum=0
        for event in self.eventList:
            nb = nb+1
            sum+= event.duration()

        if (nb == 0):
            return None
        else:
            return sum/nb

    def getStandardDeviationEventLength(self):
        sd=0
        durationList=[]
        for event in self.eventList:
            durationList.append( event.duration() )

        sd = np.std( durationList )
        return sd

    def getMaxEventLength(self):
        maxi=0
        for event in self.eventList:
            maxi=max(maxi,event.duration())
        return maxi

    def getMinEventLength(self):
        mini=3*oneDay
        for event in self.eventList:
            mini=min(mini,event.duration())
        return mini

    def getTotalLength(self):
        sum = 0
        for event in self.eventList:
            sum += event.duration()
        return sum

    def getTotalDistance(self):
        dist = 0
        for event in self.eventList:
            dist+=event.getDistance()
        return dist

    def plotEventDurationDistributionHist(self , nbBin = 10 ):
        data = []
        for event in self.eventList:
            data.append(event.duration())
        #ax = plt.subplots()
        plt.hist(data, nbBin)
        plt.xlabel( "Event duration" )
        plt.ylabel( "Number of events" )
        plt.title( "Hist. of duration of event {}".format( self.eventName ) )
        plt.figtext(.5,.85,"Number of bin: {}".format(nbBin) ,fontsize=10,ha='center')
        plt.show()

    def plotEventDurationDistributionBar(self ,minDuration = None, maxDuration = None ):
        data =[]

        for event in self.eventList:
            if ( minDuration != None):
                if ( event.duration() < minDuration ):
                    continue
            if ( maxDuration != None):
                if ( event.duration() > maxDuration ):
                    continue
            data.append( event.duration() )

        data= sorted( data )
        fig, ax = plt.subplots()
        indexes = np.arange( len(data) )
        plt.bar( indexes,data,1 )
        #plt.bar( 3,11,1 )

        plt.xlabel("Events (sorted by duration)")
        plt.ylabel("Duration")
        plt.title("Duration of events")
        plt.figtext(.5,.85,"Number of events: {} (minDuration={}, maxDuration={})".format(len(data), minDuration, maxDuration),fontsize=10,ha='center')
        plt.show()

    def plotTimeLine(self):

        y =[]
        start =[]
        end=[]

        longestEvent = None

        for event in self.eventList:
            y.append( 1 )
            start.append( event.startFrame )
            end.append( event.endFrame )

            if ( longestEvent == None ):
                longestEvent = event

            if ( event.duration() > longestEvent.duration() ):
                longestEvent = event

        fig, ax = plt.subplots()
        ax.hlines(y, start, end, 'b', lw=4)
        ax.text(0,1.01,"TimeLine of {}".format( self.eventNameWithId ) ,fontsize=10,ha='left')
        if (longestEvent != None ):
            ax.annotate('Longest event ({})'.format(longestEvent.duration()), xy=(longestEvent.startFrame, 1), xytext=(longestEvent.startFrame, 0.95), arrowprops=dict(facecolor='black', shrink=0.05))
        #fig.show()
        plt.show()

    def endRebuildEventTimeLine(self, connection, deleteExistingEvent = False):
        '''
        Delete the old event timeline and save the new calculated one in the lmtanalysis
        '''

        # if (len( self.eventList) == 0):
        #     print("no event")
        # else:
        #     print ( "Number of event: " , len( self.eventList ) )
        #     print ( "Mean length of event: " , self.getMeanEventLength() )
        #     print ( "first event frame: " , self.eventList[0].startFrame )


        if deleteExistingEvent:
            # print ( "Delete old entry in base: " + self.eventName )
            deleteEventTimeLineInBase(connection, self.eventName, self.idA, self.idB, self.idC, self.idD )
        # else:
        #     print( "Keep previous entry.")

        # print ( "Saving timeLine: " + self.eventName + " ( " + str(len( self.eventList)) + " events )")
        self.saveTimeLine(connection)

    def deleteEventTimeLineInBase( self , connection ):
         deleteEventTimeLineInBase(connection, self.eventName, self.idA, self.idB, self.idC, self.idD )

    def getDensityEventInTimeBin(self, tmin=0, tmax=None, binSize=1*oneMinute):
        '''
        compute the proportion of frames within the time bin that are included in one event type
        '''
        if ( tmax == None ):
            tmax = self.getMaxT()

        dicEvent = self.getDictionnary()

        densityEventInBinList = []

        frame = tmin

        while (frame <= tmax):

            durationEventInBin = 0

            for t in range(frame, frame + binSize ):
                if (t in dicEvent.keys()):
                    durationEventInBin = durationEventInBin + 1

            densityEventInBin = durationEventInBin/binSize

            densityEventInBinList.append(densityEventInBin)

            frame = frame + binSize

        return densityEventInBinList


    def shiftInTime( self, nbFrame ):
        '''
        Shifts all events of nbFrames
        '''
        for event in self.eventList:
            event.shift( nbFrame )


    def getLengthDistanceWithTimeLine( self, timeLineCandidate ):

        '''
        provides correlation of current event considering overlap with candidate event.
        '''

        foundEventList = []

        # print("correlation started")
        chrono = Chronometer( "getLengthDistanceWithTimeLine " + timeLineCandidate.eventName )

        maxT = self.getMaxT( )
        dico = self.getDictionnary( 0 , maxT )
        dicoCandidate = timeLineCandidate.getDictionnary( 0 , maxT )

        mergedDico = {}

        for k in dico.keys():
            if ( k in dicoCandidate ):
                mergedDico[k] = True

        mergedTimeLine = EventTimeLine( None, eventName="Merged" , loadEvent=False )
        mergedTimeLine.reBuildWithDictionnary( mergedDico )

        nbEvent = len( self.getEventList() )
        #nbMatch = len( newEvent.getEventList() )

        relativityDico = {} # relativity provide an histogram of the relative location of the even compared to the candidate. 0 is beginning, 1 is end
        for i in range(100): # zero fill
            relativityDico[i] = 0

        nbMatch = 0
        for event in self.eventList:

            for eventCandidate in mergedTimeLine.eventList:

                if event.overlapEvent( eventCandidate ):

                    nbMatch +=1
                    meanEventT = event.startFrame + (event.endFrame - event.startFrame )/2
                    foundEvent = timeLineCandidate.getEventAt( meanEventT )
                    if ( foundEvent != None ):
                        foundEventList.append( foundEvent )

                        if ( foundEvent.duration() > event.duration() * 2 ):

                            try:
                                a = ( meanEventT - foundEvent.startFrame ) / foundEvent.duration()
                                a = int( a * 100 )
                                if ( a in relativityDico ):
                                    relativityDico[a]+=1
                                else:
                                    relativityDico[a]=1

                            except:
                                pass

                    break

        # print ( "NB match with " + timeLineCandidate.eventName + " " + str(nbMatch) + " / " + str(nbEvent) + " time to compute: " + str(chrono.getTimeInMS()) )

        v=0

        if nbMatch!=0:
            v = nbMatch / nbEvent

        return [v, foundEventList,relativityDico]

    def updateMetaData( self , connection , commit= True ):
        for event in self.eventList:
            event.updateMetaData( connection )
        if commit == True:
            connection.commit()


def deleteEventTimeLineInBase( connection, eventName, idA=None, idB=None, idC=None, idD=None ):
    '''
    delete an event in dataBase
    '''

    cursor = connection.cursor()
    # print ( "Deleting event {} (idA:{},idB:{},idC:{},idD:{}) from base...".format( eventName , idA, idB, idC, idD ) )
    query = "DELETE FROM EVENT WHERE NAME=\"{0}\"".format( eventName )
    if ( idA != None ):
        query += " AND IDANIMALA={0}".format( idA )

    if ( idB != None ):
        query += " AND IDANIMALB={0}".format( idB )

    if ( idC != None ):
        query += " AND IDANIMALC={0}".format( idC )

    if ( idD != None ):
        query += " AND IDANIMALD={0}".format( idD )

    # print(query)

    cursor.execute( query )
    connection.commit()
    # print( "Number of event deleted: {} ".format ( cursor.rowcount ) )

def plotMultipleTimeLine( timeLineList , colorList=None , show=True , minValue=0 ):
    '''
    Plot multiple timeLine
    '''

    yOffset = len( timeLineList ) -1
    maxX = 0

    plt.figure( figsize=(10, yOffset/2 ) )

    for timeLine in timeLineList:
        start = []
        end = []
        y = []
        if timeLine != None:

            for event in timeLine.eventList:
                y.append( 1 + yOffset )
                start.append( event.startFrame )
                end.append( event.endFrame )
                plt.Rectangle( ( event.startFrame, 1+yOffset )  , event.duration( ) , 1 )
                if ( event.endFrame > maxX ):
                    maxX = event.endFrame
            color = "k"
            if( colorList!= None):
                color = colorList[ timeLineList.index( timeLine ) ]

            plt.hlines( y, start, end, color, lw=20)
            plt.text( minValue ,1.0 + yOffset,"{}   ".format( timeLine.eventNameWithId ) ,fontsize=9,ha='right',va='center',label='test')

        yOffset-=1

        #plt.annotate('Longest event ({})'.format( longestEvent.duration()), xy=( longestEvent.startFrame, 1), xytext=(longestEvent.startFrame, 0.95), arrowprops=dict(facecolor='black', shrink=0.05))

        #ax = plt.gca()
        #ax.relim()
    plt.grid( True , axis='x' )
    marginTop = 0.3
    plt.ylim( (0+marginTop , len( timeLineList ) +1-marginTop ) )
    plt.axes().get_yaxis().set_visible( False )
    plt.subplots_adjust( left=0.2 )

    plt.legend()

    if ( show ):
        plt.show()



class TestEventTimeLine ( unittest.TestCase ):

    def test_MergeCloseEvents(self):

        myEventTimeLine = EventTimeLine( None, "testEvent" , 1 , 2 , loadEvent=False )
        myEventTimeLine.addEvent( Event( 50,55 ) )
        myEventTimeLine.addEvent( Event( 60,65 ) )

        myEventTimeLine.addEvent( Event( 150,155 ) )
        myEventTimeLine.addEvent( Event( 160,165 ) )

        myEventTimeLine.addEvent( Event( 250,255 ) )
        myEventTimeLine.addEvent( Event( 261,265 ) )
        # xxxxxXooooX
        # 5    5    6
        # 0    5    0
        myEventTimeLine.mergeCloseEvents( 4 )
        myEventTimeLine.printEventList()
        print("---")

        result = ( myEventTimeLine.getNbEvent() == 4 )
        self.assertEqual( result, True )

    def test_PuntualEvent(self):

        myEventTimeLine = EventTimeLine( None, "testEvent" , 1 , 2 , loadEvent=False )
        myEventTimeLine.addPunctualEvent( 500 )
        myEventTimeLine.printEventList()

        result = ( myEventTimeLine.getMinT() == 500 and myEventTimeLine.getMaxT() == 500 )
        self.assertEqual( result, True )

    def test_PuntualEvent2(self):

        myEventTimeLine = EventTimeLine( None, "testEvent" , 1 , 2 , loadEvent=False )
        myEventTimeLine.addEvent( Event( 51,99 ) )
        myEventTimeLine.addPunctualEvent( 100 )
        myEventTimeLine.addPunctualEvent( 50 )

        myEventTimeLine.printEventList()
        result = ( myEventTimeLine.getMinT() == 50 and myEventTimeLine.getMaxT() == 100 )
        self.assertEqual( result, True )

    def test_PuntualEvent3(self):

        myEventTimeLine = EventTimeLine( None, "testEvent" , 1 , 2 , loadEvent=False )
        myEventTimeLine.addEvent( Event( 100,200 ) )
        myEventTimeLine.addEvent( Event( 50,99 ) )

        myEventTimeLine.printEventList()
        result = ( myEventTimeLine.getMinT() == 50 and myEventTimeLine.getMaxT() == 200 )
        self.assertEqual( result, True )

    def test_PuntualEvent4(self):

        myEventTimeLine = EventTimeLine( None, "testEvent" , 1 , 2 , loadEvent=False )
        myEventTimeLine.addEvent( Event( 100,200 ) )
        myEventTimeLine.addEvent( Event( 201,300 ) )

        myEventTimeLine.printEventList()
        result = ( myEventTimeLine.getMinT() == 100 and myEventTimeLine.getMaxT() == 300 )
        self.assertEqual( result, True )

    def test_PuntualEvent5(self):

        myEventTimeLine = EventTimeLine( None, "testEvent" , 1 , 2 , loadEvent=False )
        myEventTimeLine.addEvent( Event( 100,200 ) )
        #print("test overlap")
        myEventTimeLine.addEvent( Event( 150,250 ) )
        myEventTimeLine.addEvent( Event( 50,160 ) )
        myEventTimeLine.addEvent( Event( 50,400 ) )

        myEventTimeLine.printEventList()
        result = ( myEventTimeLine.getMinT() == 50 and myEventTimeLine.getMaxT() == 400 and len( myEventTimeLine.eventList ) == 1 )
        self.assertEqual( result, True )

    def test_PuntualEvent6(self):

        myEventTimeLine = EventTimeLine( None, "testEvent" , 1 , 2 , loadEvent=False )
        myEventTimeLine.addEvent( Event( 100,200 ) )
        #print("test overlap")
        myEventTimeLine.addEvent( Event( 202,300 ) )
        myEventTimeLine.addPunctualEvent( 201 )

        myEventTimeLine.printEventList()
        result = ( myEventTimeLine.getMinT() == 100 and myEventTimeLine.getMaxT() == 300 and len( myEventTimeLine.eventList ) == 1 )
        self.assertEqual( result, True )


if __name__ == '__main__':
    unittest.main()
